;################################################################################
;#										#
;# libgps - NMEA read library							#
;# copyright (c) 2008 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;# This library is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU Lesser General Public			#
;# License as published by the Free Software Foundation; either			#
;# version 2 of the License, or (at your option) any later version.		#
;#										#
;# This library is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU		#
;# Lesser General Public License for more details.				#
;#										#
;# You should have received a copy of the GNU Lesser General Public		#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;------------------------------------------------------------------------------
;read RMC record from serial and store data to Y...
;Nullterminierte Strings:
;Y+0	Uhrzeit hh:mm:ss
;Y+10	Breite DD.ddddddN
;Y+22	Länge DDD.ddddddE
;Y+35	Geschwindigkeit kkk in km/h
;Y+39 	Richtung in Grad DDD
;Y+43	Datum dd.mm.yy
;Byte-Values
;Y+9		data valid 0/1
;Y+52		1=breite+100grad
;Y+53-55	LMH breite binär (1/100 000 Grad)
;Y+56		1=länge+100grad
;Y+57-59	LMH länge binär (1/100 000 Grad)
;Y+60		speed
;------------------------------------------------------------------------------
libgps_getrmc:	push	ZL
		push	ZH
		push	XL
		push	XH
		push	tempreg1
		push	tempreg2
		push	tempreg3
		push	tempreg4

libgps_grmc_01:	ldi	ZL,LOW(libgps_initstr*2)	;header compare table
		ldi	ZH,HIGH(libgps_initstr*2)

		ldi	XL,7			;chars in header	
libgps_grmc_02:	libmio_gserb			;get char
		lpm	tempreg2,Z+		;get comp value
		cpi	tempreg1,0x1b		;no key?
		brne	libgps_grmc_03		;yes
		rjmp	libgps_grmc_e		;exit without data
		
libgps_grmc_03:	cp	tempreg1,tempreg2
		brne	libgps_grmc_01		;not equal
		dec	XL
		brne	libgps_grmc_02		;loop
		
;now we are at the start of the RMC message		
;read time
		ldi	XL,':'			;separator
		rcall	libgps_getchar		;get h*10
		st	Y,tempreg1		;store
		rcall	libgps_getchar		;get h
		std	Y+1,tempreg1		;store
		std	Y+2,XL			;store delimiter
		rcall	libgps_getchar		;get m*10
		std	Y+3,tempreg1		;store
		rcall	libgps_getchar		;get m
		std	Y+4,tempreg1		;store		
		std	Y+5,XL			;store delimiter
		rcall	libgps_getchar		;get s*10
		std	Y+6,tempreg1		;store
		rcall	libgps_getchar		;get s
		std	Y+7,tempreg1		;store		
		std	Y+8,const_0		;store end				

		
;read valid flag
		rcall	libgps_waitc		;wait for comma
		rcall	libgps_getchar		;valid flag
		cpi	tempreg1,'A'		;is valid?
		breq	libgps_grmc_06		;yes	
		
		ldi	XL,6			;7 values to be ignored
libgps_grmc_04:	rcall	libgps_waitc		;wait for comma
		dec	XL
		brne	libgps_grmc_04
		std	Y+9,const_0		;invalid
		rjmp	libgps_grmc_60		;continue with datum		
		
;------------------------------------------------------------------------------				
;get latitude and convert to DD.dddddd
;------------------------------------------------------------------------------											
libgps_grmc_06:	std	Y+9,const_1		;valid
		rcall	libgps_waitc		;wait for comma
		rcall	libgps_getchar		;get d*10
		std	Y+10,tempreg1		;store		
		rcall	libgps_getchar		;get d*1
		std	Y+11,tempreg1		;store
		ldi	XL,'.'			;delimiter
		std	Y+12,XL			;store
		movw	XL,YL			;copy ptr
		adiw	XL,13			;set offset
		ldi	tempreg2,8		;chars+2
libgps_grmc_07:	st	X+,const_0
		dec	tempreg2
		brne	libgps_grmc_07				

		ldi	ZH,HIGH(libgps_minf*2)
		ldi	ZL,LOW(libgps_minf*2)				
		movw	XL,YL			;copy result base
		adiw	XL,13			;set offset
		
				
;now get the minutes			
		rcall	libgps_addval

				
;add number base to all digits
		movw	XL,YL
		adiw	XL,13			;set offset
		ldi	tempreg2,6
libgps_grmc_08:	ld	tempreg1,X
		subi	tempreg1,0xd0
		st	X+,tempreg1
		dec	tempreg2
		brne	libgps_grmc_08
											
		rcall	libgps_waitc		;wait for comma
		rcall	libgps_getchar		;get N/S
		ldi	XL,0x20			;space
		std	Y+19,XL
		std	Y+20,tempreg1
		std	Y+21,const_0
				
;------------------------------------------------------------------------------				
;get longitude and convert to DDD.dddddd
;------------------------------------------------------------------------------											
libgps_grmc_09:	rcall	libgps_waitc		;wait for comma
		rcall	libgps_getchar		;get d*100
		cpi	tempreg1,0x30		;0
		brne	libgps_grmc9a
		ldi	tempreg1,0x20		;replace with space	
libgps_grmc9a:	std	Y+22,tempreg1		;store		
		rcall	libgps_getchar		;get d*10
		std	Y+23,tempreg1		;store
		rcall	libgps_getchar		;get d*1
		std	Y+24,tempreg1		;store		
		ldi	XL,'.'			;delimiter
		std	Y+25,XL			;store
		movw	XL,YL			;copy ptr
		adiw	XL,26			;set offset
		ldi	tempreg2,8		;chars+2
libgps_grmc_10:	st	X+,const_0
		dec	tempreg2
		brne	libgps_grmc_10				

		ldi	ZH,HIGH(libgps_minf*2)	;pointer to factor table
		ldi	ZL,LOW(libgps_minf*2)
		movw	XL,YL			;result base
		adiw	XL,26			;offset	
								
;now the minutes		
		rcall	libgps_addval		;get minutes
					
;add number base to all digits
		movw	XL,YL
		adiw	XL,26			;set offset
		ldi	tempreg2,6
libgps_grmc_11:	ld	tempreg1,X
		subi	tempreg1,0xd0
		st	X+,tempreg1
		dec	tempreg2
		brne	libgps_grmc_11
											
		rcall	libgps_waitc		;wait for comma
		rcall	libgps_getchar		;get E/W
		ldi	XL,0x20			;space
		std	Y+32,XL
		std	Y+33,tempreg1
		std	Y+34,const_0
		
;------------------------------------------------------------------------------				
;get speed and convert to km/h
;------------------------------------------------------------------------------		
libgps_grmc_40:	rcall	libgps_waitc		;next
		ldi	tempreg3,0		;act pre
		ldi	tempreg4,0		;act past
libgps_grmc_41:	rcall	libgps_getchar		;get char
		cpi	tempreg1,','		;end
		breq	libgps_grmc_43		;yes
		cpi	tempreg1,'.'		;sep
		breq	libgps_grmc_42		;yes			
		subi	tempreg1,0x30
		ldi	XL,10
		mul	XL,tempreg3
		mov	tempreg3,r0
		add	tempreg3,tempreg1
		rjmp	libgps_grmc_41
libgps_grmc_42:	rcall	libgps_getchar		;get char
		cpi	tempreg1,','		;end
		breq	libgps_grmc_43		;yes
		subi	tempreg1,0x30
		mov	tempreg4,tempreg1	;copy
;data is now available
libgps_grmc_43:	ldi	XL,185			;factor
		mul	XL,tempreg3		;get KMH*100 to r0/r1
		movw	ZL,r0			;store
		ldi	XL,19			;factor/10
		mul	XL,tempreg4
		add	ZL,r0
		adc	ZH,r1
		
		
libgps_grmc_44:	ldi	XH,HIGH(10000)
		ldi	XL,LOW(10000)
		rcall	libgps_bdc		;convert 100.00 kmh
		std	Y+35,tempreg4
		ldi	XH,HIGH(1000)
		ldi	XL,LOW(1000)
		rcall	libgps_bdc		;convert 10.00 kmh
		std	Y+36,tempreg4			
		ldi	XH,HIGH(100)
		ldi	XL,LOW(100)
		rcall	libgps_bdc		;convert 1.00 kmh
		std	Y+37,tempreg4			
		std	Y+38,const_0		;output stopper
		cpi	tempreg1,','
		
		breq	libgps_grmc50a
						
;------------------------------------------------------------------------------				
;get angle
;------------------------------------------------------------------------------								
libgps_grmc_50:	rcall	libgps_waitc		;wait for comma
libgps_grmc50a:
libgps_grmc50b:	ldi	ZH,0x30
		ldi	ZL,0x30
		ldi	XL,0x30
		std	Y+39,ZH
		std	Y+40,ZL
		std	Y+41,XL
		std	Y+42,const_0		
libgps_grmc_51:	rcall	libgps_getchar		;get char	
		cpi	tempreg1,'.'
		breq	libgps_grmc_60
		cpi	tempreg1,','
		breq	libgps_grmc60a		
		mov	ZH,ZL
		mov	ZL,XL
		mov	XL,tempreg1
		std	Y+39,ZH
		std	Y+40,ZL
		std	Y+41,XL
		rjmp	libgps_grmc_51
				
;------------------------------------------------------------------------------				
;get date
;------------------------------------------------------------------------------							
libgps_grmc_60:	rcall	libgps_waitc		;wait for comma
libgps_grmc60a:	ldi	XL,'.'			;separator
		rcall	libgps_getchar		;get d*10
		std	Y+43,tempreg1		;store
		rcall	libgps_getchar		;get d
		std	Y+44,tempreg1		;store
		std	Y+45,XL			;store delimiter
		rcall	libgps_getchar		;get m*10
		std	Y+46,tempreg1		;store
		rcall	libgps_getchar		;get m
		std	Y+47,tempreg1		;store		
		std	Y+48,XL			;store delimiter
		rcall	libgps_getchar		;get y*10
		std	Y+49,tempreg1		;store
		rcall	libgps_getchar		;get y
		std	Y+50,tempreg1		;store		
		std	Y+51,const_0		;store end		


;------------------------------------------------------------------------------				
;calc bin values for long & lat
;------------------------------------------------------------------------------							
libgps_grmc_70:	push	YH
		push	YL
		movw	XL,YL
		adiw	XL,9
		adiw	YL,52
		rcall	libgps_decbin
		pop	YL
		pop	YH

		push	YH
		push	YL
		movw	XL,YL
		adiw	XL,22
		adiw	YL,58
		rcall	libgps_decbin
		pop	YL
		pop	YH
		

libgps_grmc_e:	pop	tempreg4		
		pop	tempreg3
		pop	tempreg2
		pop	tempreg1
		pop	XH
		pop	XL
		pop	ZH
		pop	ZL
		ret

;------------------------------------------------------------------------------
; bin-Dec
;------------------------------------------------------------------------------			
libgps_bdc:	ldi	tempreg4,0x30
libgps_bdc1:	cp	ZL,XL
		cpc	ZH,XH
		brcs	libgps_bdc2		
		sub	ZL,XL
		sbc	ZH,XH
		inc	tempreg4
		rjmp	libgps_bdc1
			
libgps_bdc2:	ret		
				
;------------------------------------------------------------------------------
; add tempreg1 * (Z) to (X)
;------------------------------------------------------------------------------				
libgps_addval:	push	XL		;save registers
		push	XH
		push	YL

		ldi	ZH,HIGH(libgps_minf*2)	;pointer to factor table
		ldi	ZL,LOW(libgps_minf*2)
		ldi	YL,6		;1 number of chars to do
				
libgps_addval0:	rcall	libgps_getchar	;n get num char
		cpi	tempreg1,'.'	;ignore decimal point
		breq	libgps_addval0
		subi	tempreg1,0x30	;1 sub number base
		adiw	XL,6		;2 set pointer beyond last
		clr	tempreg3	;1 clear overflow
		ldi	tempreg2,6	;1 number
libgps_addval1:	lpm	r0,Z+		;3 get factor		
		mul	r0,tempreg1	;2 mult with n
		ld	tempreg4,-X	;2 get result byte
		add	tempreg4,r0		;1 add
		add	tempreg4,tempreg3 	;1 add overflow
		clr	tempreg3	;1 clear overflow
libgps_addval2:	cpi	tempreg4,10	;1 compare
		brcs	libgps_addval3	;1/2
		subi	tempreg4,10	;1 result -10
		subi	tempreg3,0xff	;1 overflow+1
		rjmp	libgps_addval2	;2 loop			
libgps_addval3:	st	X,tempreg4
		dec	tempreg2
		brne	libgps_addval1
		dec	YL		;1 char counter
		brne	libgps_addval0	;loop
		pop	YL		;restore registers
		pop	XH
		pop	XL
		ret			;thats all

;------------------------------------------------------------------------------
; get one char from serial
;------------------------------------------------------------------------------		
libgps_getchar:	libmio_gser
		ret

;wait for comma
libgps_waitc:	rcall	libgps_getchar
		cpi	tempreg1,','
		brne	libgps_waitc
		ret						
		
;intro data
libgps_initstr:	.db "$GPRMC,",0
libgps_minf:	.db 7,6,6,6,6,1
		.db 7,6,6,6,1,0
		.db 7,6,6,1,0,0
		.db 7,6,1,0,0,0
		.db 7,1,0,0,0,0
		.db 2,0,0,0,0,0

.include	"libgps/bindec.asm"		