;################################################################################
;#										#
;# libi2c - i2c library for ATmega controllers 					#
;# copyright (c) 2007-2008 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 2		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; withsts even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;-------------------------------------------------------------------------------
; initialize twi
;-------------------------------------------------------------------------------
libi2c_initl:	cbi	portd,6
		ldi	tempreg1,92		;set bitrate
		sts	TWBR,tempreg1
		ldi	tempreg1,(1<<TWEN) | (1<<TWINT)	;enable twi
		sts	TWCR,tempreg1

;-------------------------------------------------------------------------------
; generate stop condition
;-------------------------------------------------------------------------------
libi2c_stop:	ldi	tempreg1,(1<<TWSTO) | (1<<TWINT) | (1<<TWEN)
		sts	TWCR,tempreg1			;set control
		nop
		nop
libi2c_stop_1:	lds	tempreg1,TWCR			;get control
		
		ldi	tempreg1,80
libi2c_stop_2:	dec	tempreg1
		brne	libi2c_stop_2
		ret		

;-------------------------------------------------------------------------------
; generate start condition
;-------------------------------------------------------------------------------
libi2c_start:	ldi	tempreg1,(1<<TWSTA) | (1<<TWINT) | (1<<TWEN)
		rcall	libi2c_wready
		ldi	tempreg1,40
libi2c_start_1:	dec	tempreg1
		brne	libi2c_start_1
		ret		
			
;-------------------------------------------------------------------------------
; write byte (tempreg2) to I2C
;-------------------------------------------------------------------------------
libi2c_wbyte:	sts	TWDR,tempreg2
		ldi	tempreg1,(1<<TWINT) | (1<<TWEN)
		rjmp	libi2c_wready

;-------------------------------------------------------------------------------
; read byte (tempreg2) from I2C (ACK)
;-------------------------------------------------------------------------------
libi2c_rbyte:	ldi	tempreg1,(1<<TWINT) | (1<<TWEN) | (1<<TWEA)
		rcall	libi2c_wready
		lds	tempreg2,TWDR			;get data
		ret

;-------------------------------------------------------------------------------
; read byte (tempreg2) from I2C (NACK)
;-------------------------------------------------------------------------------
libi2c_rbyten:	ldi	tempreg1,(1<<TWINT) | (1<<TWEN)
		rcall	libi2c_wready
		lds	tempreg2,TWDR			;get data
		ret

;-------------------------------------------------------------------------------
; set control to tempreg1 and wait for TWI is ready
;-------------------------------------------------------------------------------
libi2c_wready:	sts	TWCR,tempreg1			;set control
libi2c_wready1:	lds	tempreg1,TWCR			;get control
		sbrs	tempreg1,TWINT			;skip if ready
		rjmp	libi2c_wready1			;loop
		lds	tempreg3,TWSR			;get status
		andi	tempreg3,0xf8			;mask bits		
		ret					;thats all

;-------------------------------------------------------------------------------
; write memory address X to I2C EEPROM (1)
;-------------------------------------------------------------------------------
libi2c_wad:	rcall	libi2c_start		;start condition
		cpi	tempreg3,0x08
		brne	libi2c_wad_e		;error
		ldi	tempreg2,0xa2		;address+W
		rcall	libi2c_wbyte		
		cpi	tempreg3,0x18
		brne	libi2c_wad_e		;error
libi2c_wad_1:	mov	tempreg2,XH		;write address MSB
		rcall	libi2c_wbyte		
		cpi	tempreg3,0x28
		brne	libi2c_wad_e		;error
		mov	tempreg2,XL		;write address LSB
		rcall	libi2c_wbyte		
		cpi	tempreg3,0x28
		brne	libi2c_wad_e		;error
		clr	ereg
		ret
libi2c_wad_e:	ldi	ereg,15			;set error
		rjmp	libi2c_stop		;stop condition and end

;-------------------------------------------------------------------------------
; set EEPROM (1) to read
;-------------------------------------------------------------------------------
libi2c_srd:	rcall	libi2c_start		;start condition
		cpi	tempreg3,0x10
		brne	libi2c_wad_e		;error
		ldi	tempreg2,0xa3		;address+R
		rcall	libi2c_wbyte		
		cpi	tempreg3,0x40
		brne	libi2c_wad_e		;error
		clr	ereg
		ret
					
;-------------------------------------------------------------------------------
; read data byte X+ from external eeprom (tempreg4) to tempreg2
;-------------------------------------------------------------------------------
.ifdef use_libi2c_read
libi2c_read:	rcall	libi2c_wad		;write address	
		cpi	ereg,0			;error?
		brne	libi2c_read1		;yes
		rcall	libi2c_srd		;set to read
		cpi	ereg,0			;error?
		brne	libi2c_read1
		rcall	libi2c_rbyten		;read byte with NACK
		cpi	tempreg3,0x58		;error?
		brne	libi2c_read1
		adiw	XL,1			;X+1
		clr	ereg
		rjmp	libi2c_stop
		
libi2c_read1:	ldi	ereg,15			;I2C error		
		rjmp	libi2c_stop
.endif

;-------------------------------------------------------------------------------
; write data byte tempreg2 to external eeprom (tempreg4) address X+
;-------------------------------------------------------------------------------
.ifdef use_libi2c_write
libi2c_write:	push	tempreg2		;save data
		rcall	libi2c_wad		;write address	
		pop	tempreg2		;restore data
		cpi	ereg,0			;error?
		brne	libi2c_write1
		rcall	libi2c_wbyte		;write byte
		cpi	tempreg3,0x28		;error?
		brne	libi2c_write1
		adiw	XL,1			;Y+1
		clr	ereg
		rjmp	libi2c_stop

libi2c_write1:	ldi	ereg,15			;I2C error		
		rjmp	libi2c_stop
.endif



;-------------------------------------------------------------------------------
; read temperature from external lm75 sensor no. (tempreg4) to X
;-------------------------------------------------------------------------------
.ifdef use_libi2c_lm75
libi2c_rlm75:	rcall	libi2c_start		;start condition
		cpi	tempreg3,0x08
		brne	libi2c_rlm75e		;error
		mov	tempreg2,tempreg4	;sensor number		
		andi	tempreg2,0x07		;8 addresses
		lsl	tempreg2		;set correct devsel
		ori	tempreg2,0x91
		rcall	libi2c_wbyte
		cpi	tempreg3,0x40		;error?
		brne	libi2c_rlm75e
		rcall	libi2c_rbyte		;read byte with ACK
		cpi	tempreg3,0x50		;error?
		brne	libi2c_rlm75e
		mov	XL,tempreg2		;get low byte
		rcall	libi2c_rbyten		;read byte with NAK
		cpi	tempreg3,0x58		;error?
		brne	libi2c_rlm75e
		clr	XH			;high result
		lsl	tempreg2		; result * 2
		rol	XL
		brcc	libi2c_rlm751
		com	XH
libi2c_rlm751:	clr	ereg			;no error
		rjmp	libi2c_stop		;stop and end
		
libi2c_rlm75e:	ldi	ereg,15			;I2C error		
		rjmp	libi2c_stop
.endif

;-------------------------------------------------------------------------------
; subroutines
;-------------------------------------------------------------------------------
.ifdef use_libi2c_rtc32
libi2c_r32ncv:	mov	XL,tempreg2
		swap	XL
		andi	XL,0x0f
		subi	XL,0xd0			;add ascii base
		st	Y+,XL
		mov	XL,tempreg2
		andi	XL,0x0f
		subi	XL,0xd0			;add ascii base
		st	Y+,XL
		ret

libi2c_r32bcv:	ld	XL,Y+
		subi	XL,0x30
		swap	XL
		andi	XL,0xf0
		mov	tempreg2,XL
		ld	XL,Y+
		subi	XL,0x30
		andi	XL,0x0f
		or	tempreg2,XL
		ret

libi2c_r32sad:	rcall	libi2c_start		;start condition
		cpi	tempreg3,0x08
		brne	libi2c_r32err		;error
		ldi	tempreg2,0xd0		;write
		rcall	libi2c_wbyte		
		cpi	tempreg3,0x18
		brne	libi2c_r32err		;error		
		mov	tempreg2,tempreg4	;read devsel
		rcall	libi2c_wbyte		
		cpi	tempreg3,0x28
		brne	libi2c_r32err		;error
		clr	ereg
		rjmp	libi2c_stop
		ret

libi2c_r32sadn:	rcall	libi2c_start		;start condition
		cpi	tempreg3,0x08
		brne	libi2c_r32err		;error
		ldi	tempreg2,0xd0		;write
		rcall	libi2c_wbyte		
		cpi	tempreg3,0x18
		brne	libi2c_r32err		;error		
		mov	tempreg2,tempreg4	;address
		rcall	libi2c_wbyte		
		cpi	tempreg3,0x28
		brne	libi2c_r32err		;error
		clr	ereg
		ret

		
libi2c_r32err:	ldi	ereg,15			;I2C error
		libmio_sync		
		rjmp	libi2c_stop

;-------------------------------------------------------------------------------
; read time from ds3232 rtc to Y+
;-------------------------------------------------------------------------------
libi2c_r32rt:	clr	tempreg4		;address
		rcall	libi2c_r32sad
		cpi	ereg,0
		brne	libi2c_r32err
		libmio_sync
		libmio_sync
	    	rcall	libi2c_start		;start condition
		cpi	tempreg3,0x08
		brne	libi2c_r32err		;error
		ldi	tempreg2,0xd1		;address read
		rcall	libi2c_wbyte
		cpi	tempreg3,0x40		;error?
		brne	libi2c_r32err		;error
		rcall	libi2c_rbyte		;get first byte
		push	tempreg2
		rcall	libi2c_rbyte		;get first byte
		push	tempreg2
		rcall	libi2c_rbyten
		rcall	libi2c_r32ncv
		ldi	XL,':'
		st	Y+,XL
		pop	tempreg2
		rcall	libi2c_r32ncv
		ldi	XL,':'
		st	Y+,XL
		pop	tempreg2
		rcall	libi2c_r32ncv
		st	Y+,const_0
		rjmp	libi2c_stop
		
;-------------------------------------------------------------------------------
; read date from ds3232 rtc to Y+
;-------------------------------------------------------------------------------
libi2c_r32rd:	ldi	tempreg4,4		;address
		rcall	libi2c_r32sad
		cpi	ereg,0
		brne	libi2c_r32err
		libmio_sync
		libmio_sync
		rcall	libi2c_start		;start condition
		cpi	tempreg3,0x08
		brne	libi2c_r32err		;error
		ldi	tempreg2,0xd1		;address read
		rcall	libi2c_wbyte
		cpi	tempreg3,0x40		;error?
		brne	libi2c_r32err		;error
		rcall	libi2c_rbyte		;get first byte
		rcall	libi2c_r32ncv
		ldi	XL,'.'
		st	Y+,XL
		rcall	libi2c_rbyte
		rcall	libi2c_r32ncv
		ldi	XL,'.'
		st	Y+,XL
		rcall	libi2c_rbyten
		rcall	libi2c_r32ncv
		st	Y+,const_0
		rjmp	libi2c_stop

;-------------------------------------------------------------------------------
; write time from Y+ to ds3232
;-------------------------------------------------------------------------------
libi2c_r32wt:	push	tempreg5
		push	tempreg6
		push	tempreg7
		rcall	libi2c_r32bcv		
		mov	tempreg7,tempreg2	;copy h
		rcall	libi2c_r32bcv		
		mov	tempreg6,tempreg2	;copy m
		rcall	libi2c_r32bcv		
		mov	tempreg5,tempreg2	;copy s
		clr	tempreg4
		rcall	libi2c_r32sadn		;write adr
		cpi	ereg,0
		brne	libi2c_r32wte
		mov	tempreg2,tempreg5	;s
		rcall	libi2c_wbyte
		mov	tempreg2,tempreg6	;m
		rcall	libi2c_wbyte
		mov	tempreg2,tempreg7	;h
		rcall	libi2c_wbyte
		rcall	libi2c_stop
libi2c_r32wte:	pop	tempreg7
		pop	tempreg6
		pop	tempreg5
		ret

;-------------------------------------------------------------------------------
; write date from Y+ to ds3232
;-------------------------------------------------------------------------------
libi2c_r32wd:	push	tempreg5
		push	tempreg6
		push	tempreg7
		rcall	libi2c_r32bcv		
		mov	tempreg7,tempreg2	;copy d
		rcall	libi2c_r32bcv		
		mov	tempreg6,tempreg2	;copy m
		rcall	libi2c_r32bcv		
		mov	tempreg5,tempreg2	;copy y
		ldi	tempreg4,4
		rcall	libi2c_r32sadn		;write adr
		cpi	ereg,0
		brne	libi2c_r32wte
		mov	tempreg2,tempreg7	;d
		rcall	libi2c_wbyte
		mov	tempreg2,tempreg6	;m
		rcall	libi2c_wbyte
		mov	tempreg2,tempreg5	;y
		rcall	libi2c_wbyte
		rcall	libi2c_stop
		rjmp	libi2c_r32wte

;-------------------------------------------------------------------------------
; read alarm (hh:mm) from ds3232 rtc to Y+
;-------------------------------------------------------------------------------
libi2c_r32ra:	ldi	tempreg4,8		;address
		rcall	libi2c_r32sad
		cpi	ereg,0
		brne	libi2c_r32err1
		libmio_sync
		libmio_sync
	    	rcall	libi2c_start		;start condition
		cpi	tempreg3,0x08
		brne	libi2c_r32err1		;error
		ldi	tempreg2,0xd1		;address read
		rcall	libi2c_wbyte
		cpi	tempreg3,0x40		;error?
		brne	libi2c_r32err1		;error
		rcall	libi2c_rbyte		;get first byte
		push	tempreg2
		rcall	libi2c_rbyten
		rcall	libi2c_r32ncv
		ldi	XL,':'
		st	Y+,XL
		pop	tempreg2
		rcall	libi2c_r32ncv
		st	Y+,const_0
		rjmp	libi2c_stop

;-------------------------------------------------------------------------------
; write alarm from Y+ to ds3232
;-------------------------------------------------------------------------------
libi2c_r32wa:	push	tempreg6
		push	tempreg7
		rcall	libi2c_r32bcv		
		mov	tempreg7,tempreg2	;copy h
		rcall	libi2c_r32bcv		
		mov	tempreg6,tempreg2	;copy m
		ldi	tempreg4,7
		rcall	libi2c_r32sadn		;write adr
		cpi	ereg,0
		brne	libi2c_r32wae
		mov	tempreg2,const_0	;s=0
		rcall	libi2c_wbyte
		mov	tempreg2,tempreg6	;m
		andi	tempreg2,0x7f
		rcall	libi2c_wbyte
		mov	tempreg2,tempreg7	;h
		andi	tempreg2,0x7f
		rcall	libi2c_wbyte
		ldi	tempreg2,0x80		;alarm at match
		rcall	libi2c_wbyte		
libi2c_r32wae:	rcall	libi2c_stop
		pop	tempreg7
		pop	tempreg6
		ret

libi2c_r32err1:	rjmp	libi2c_r32err

;-------------------------------------------------------------------------------
; read alarm flag and status from ds3232 rtc to X
;-------------------------------------------------------------------------------
libi2c_r32ras:	ldi	tempreg4,0x0e		;address
		rcall	libi2c_r32sad
		cpi	ereg,0
		brne	libi2c_r32err1
		libmio_sync
		libmio_sync
	    	rcall	libi2c_start		;start condition
		cpi	tempreg3,0x08
		brne	libi2c_r32err1		;error
		ldi	tempreg2,0xd1		;address read
		rcall	libi2c_wbyte
		cpi	tempreg3,0x40		;error?
		brne	libi2c_r32err1		;error
		rcall	libi2c_rbyte		;get first byte
		mov	XL,tempreg2
		rcall	libi2c_rbyten
		mov	XH,tempreg2
		rjmp	libi2c_stop



;-------------------------------------------------------------------------------
; write alarm status from X to ds3232
;-------------------------------------------------------------------------------
libi2c_r32was:	ldi	tempreg4,0x0e
		rcall	libi2c_r32sadn		;write adr
		cpi	ereg,0
		brne	libi2c_r32err1
		mov	tempreg2,XL	
		rcall	libi2c_wbyte
		mov	tempreg2,XH	
		rcall	libi2c_wbyte
		rjmp	libi2c_stop


.endif
		
