;################################################################################
;#										#
;# avr-chipbasic2m - mobile single chip basic computer with ATmega644		#
;# file functions								#
;# copyright (c) 2008 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 2		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;------------------------------------------------------------------------------
;file screen mask
;------------------------------------------------------------------------------
file_menu:	libdfl_size			;check if dataflash is available
		cpi	tempreg1,0
;		breq	file_menu_ex
		clr	tempreg2		;set menu entry to first
file_menu_re:	ldi	ZL,LOW(menudata_file*2)
		ldi	ZH,HIGH(menudata_file*2)
		libmio_copymenu		
				
file_menu_0:	ldi	r19,5
		libmio_menu5
		cpi	tempreg1,0xed		;esc
		brne	file_menu_1
file_menu_ex:	ret
		
file_menu_1:	cpi	tempreg1,0xea		;enter
		brne	file_menu_0		;no -> loop
file_menu_2:	cpi	tempreg2,0		;delete file
		brne	file_menu_3
		call	fsys_del
		clr	tempreg2
		rjmp	file_menu_re
						
file_menu_3:	cpi	tempreg2,1		
		brne	file_menu_4
		call	file_limg		;load image
		ldi	tempreg2,1
		rjmp	file_menu_re
		
file_menu_4:	cpi	tempreg2,2		
		brne	file_menu_5
		call	file_simg		;save image
		ldi	tempreg2,2
		rjmp	file_menu_re		

file_menu_5:	cpi	tempreg2,3		
		brne	file_menu_6
		call	file_sscr		;send screenshot
		ldi	tempreg2,3
		rjmp	file_menu_re						
		
		
file_menu_6:	call	fmenu_fmt		;format		
		ldi	tempreg2,4
		rjmp	file_menu_re
		
;------------------------------------------------------------------------------
;format screen
;------------------------------------------------------------------------------
fmenu_fmt:	clt				;V1
		rcall	fmenu_ask
		libmio_nokey
		libmio_waitkey
		cpi	tempreg1,0xed		;esc
		breq	fmenu_fmt_1
		cpi	tempreg1,0xea		;enter
		brne	fmenu_fmt

		set	
		rcall	fmenu_ask
		libmio_nokey
		libmio_waitkey
		cpi	tempreg1,0xe2		;left
		breq	fmenu_fmt_2

fmenu_fmt_1:	ret
fmenu_fmt_2:	libmio_oborder
		ldi	XL,22
		ldi	XH,20
		ldi	ZL,LOW(text_file_4*2)
		ldi	ZH,HIGH(text_file_4*2)		
		libmio_xztext		
		libmio_thistext
		.db	32,16,">",0
		libdfl_format
		ret		

;------------------------------------------------------------------------------
;load image
;------------------------------------------------------------------------------
file_limg:	libmio_nokey
		libmio_clrscr
		ldi	ZL,LOW(libmio_vram)		;
		ldi	ZH,HIGH(libmio_vram)
		ldi	tempreg1,0x15			;send NAK to start
		libmio_pser
		ldi	tempreg3,0			;blocks received
		ldi	XL,0				;page number
		ldi	XH,0			
		clr	ereg
			
file_limg_1:	clr	ereg
		ldi	YL,LOW(bas_array+512)		;address
		ldi	YH,HIGH(bas_array+512)
		call	xm_rxbl				;send data block
		ldi	tempreg1,0x06			;send ack
		libmio_pser
		cpi	ereg,0				;no
		breq	file_limg_4
		ret
		
file_limg_4:	clr	ereg
		ldi	YL,LOW(bas_array+640)		;address
		ldi	YH,HIGH(bas_array+640)
		call	xm_rxbl				;send data block
		cpi	ereg,0				;no
		breq	file_limg_5
		ldi	tempreg1,0x06			;send ack
		libmio_pser
		ret
		
file_limg_5:	push	XL
		push	XH
		ldi	YL,LOW(bas_array+512)		;address
		ldi	YH,HIGH(bas_array+512)
		libdfl_writeraw
		ldi	tempreg1,0x06			;send ack
		libmio_pser
		pop	XH
		pop	XL
		adiw	XL,1
		ldi	YL,0xff
		mov	YH,XL
		andi	YH,3
		cpi	YH,0x00
		brne	file_limg_1
		st	Z+,YL
		rjmp	file_limg_1
			
		
;------------------------------------------------------------------------------
;save image
;------------------------------------------------------------------------------
file_simg:	ret


;------------------------------------------------------------------------------
;send screenshot
;------------------------------------------------------------------------------
file_sscr:	ldi	tempreg1,'G'		;identify graphics
		libmio_pser
		ldi	tempreg1,'5'		;identify  mode 5
		libmio_pser
		ldi	tempreg1,'1'		;palette entry 0 = blue
		libmio_pser		
		ldi	tempreg1,'7'		;palette entry 1 = white
		libmio_pser				
		ldi	tempreg1,0x0a
		libmio_pser

file_sscr_1:	ldi	ZL,LOW(scrs_loc*2)	
		ldi	ZH,HIGH(scrs_loc*2)
		ldi	YH,64			;number of pixlines
file_sscr_2:	ldi	YL,16			;number of chars per line
file_sscr_3:	lpm	tempreg1,Z
		swap	tempreg1		;high nibble first
		rcall	file_sscr_c
		lpm	tempreg1,Z+
		rcall	file_sscr_c
		dec	YL
		brne	file_sscr_3
		ldi	tempreg1,0x0a
		libmio_pser
		dec	YH
		brne	file_sscr_2
		ret
		
file_sscr_c:	andi	tempreg1,0x0f		;mask bits
		subi	tempreg1,0xd0		;+30
		cpi	tempreg1,0x3a		;>9
		brcs	file_sscr_c1		;jump, if not
		subi	tempreg1,0xf9		;+6
file_sscr_c1:	jmp	libmio_pser		


;------------------------------------------------------------------------------
;ask for formatting
;------------------------------------------------------------------------------								
fmenu_ask:	libmio_mborder
		brtc	fmenu_ask_1
		ldi	XL,7
		ldi	XH,10
		ldi	ZL,LOW(text_file_1*2)
		ldi	ZH,HIGH(text_file_1*2)		
		libmio_xztext		
fmenu_ask_1:	ldi	XL,7
		ldi	XH,20
		ldi	ZL,LOW(text_file_2*2)	
		ldi	ZH,HIGH(text_file_2*2)		
		libmio_xztext	
		ldi	XL,7
		ldi	XH,32
		ldi	ZL,LOW(text_file_3*2)	
		ldi	ZH,HIGH(text_file_3*2)	
		brtc	fmenu_ask_2
		ldi	ZL,LOW(text_yn*2)	
		ldi	ZH,HIGH(text_yn*2)				
fmenu_ask_2:	libmio_xztext		
		ret	
		
