;################################################################################
;#										#
;# avr-chipbasic2 - single chip basic computer with ATmega644			#
;# dataflash filesystem functions						#
;# copyright (c) 2006/2007 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 2		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;------------------------------------------------------------------------------
;delete file
;------------------------------------------------------------------------------
fsys_del:	clr	tempreg4		;first file	
fsys_del_00:	rcall	fsys_check		;check for dataflash
		ldi	XL,19
		ldi	ZL,LOW(text_fsys_3*2)	;text pointer
		ldi	ZH,HIGH(text_fsys_3*2)
		libmio_mborder						
		
fsys_del_01:	rcall	fsys_select		;select a file
		brtc	fsys_del_02		;ESC		
		lds	tempreg1,libmio_fileno
		sts	partab,tempreg1
		rcall	fsys_gettype		;get type
		cpi	tempreg1,0xfc		;empty
		breq	fsys_del_01		;non existing file, ignore
fsys_adel:	libmio_oborder	
		ldi	XL,16
		ldi	XH,20
		ldi	ZL,LOW(text_fsys_4*2)	;text pointer
		ldi	ZH,HIGH(text_fsys_4*2)
		libmio_xztext				
		ldi	XL,7
		ldi	XH,32
		ldi	ZL,LOW(text_yn*2)	;text pointer
		ldi	ZH,HIGH(text_yn*2)
		libmio_xztext		
		libmio_nokey
		libmio_waitkey
		cpi	tempreg1,0xea		;enter
		brne	fsys_del_00
		lds	XL,libmio_fileno
		rcall	fsys_delete
		set				;done
fsys_del_02:	ret


;------------------------------------------------------------------------------
;save file (3K)
;------------------------------------------------------------------------------
fsys_sav3k:	libdfl_size			;check for DF
		cpi	tempreg1,0		;none
		breq	fsys_sav3k_03		;exit if no DF available

		clr	tempreg4		;first file	
fsys_sav3k_00:	rcall	fsys_check		;check for dataflash
		ldi	XL,1
		ldi	ZL,LOW(text_fsys_5*2)	;text pointer
		ldi	ZH,HIGH(text_fsys_5*2)		
		libmio_mborder		

		lds	tempreg1,libmio_lastmenu
		subi	tempreg1,0xcf
		libmio_outchar
		ldi	ZL,LOW(text_fsys_6*2)	;text pointer
		ldi	ZH,HIGH(text_fsys_6*2)
		libmio_romtext				
		
fsys_sav3k_01:	rcall	fsys_select		;select a file
		brtc	fsys_sav3k_04		;ESC		
		lds	tempreg1,libmio_fileno
		sts	partab,tempreg1
		rcall	fsys_gettype		;get type
		cpi	tempreg1,0xe0		;empty
		brcs	fsys_sav3k_01

fsys_sav3k_02:	rcall	fsys_free		;count free pages
		cpi	YH,0x00			;<256
		brne	fsys_sav3k_03
		cpi	YL,0x0c			;needed pages for a program
		brcc	fsys_sav3k_03		;we have...
		rjmp	fsys_sav3k_01		;loop
fsys_sav3k_03:	rcall	fsys_bsave		;save this file
		set				;done
		ret
		
fsys_sav3k_04:	ret				;end


;------------------------------------------------------------------------------
;save file (1,25K), gps
;------------------------------------------------------------------------------
fsys_savegps:	libdfl_size			;check for DF
		cpi	tempreg1,0		;none
		breq	fsys_savgps_04		;exit if no DF available
		clr	tempreg4		;first file	
fsys_savgps_00:	rcall	fsys_check		;check for dataflash
		ldi	ZL,LOW(text_fsys_10*2)	;text
		ldi	ZH,HIGH(text_fsys_10*2)
		ldi	XL,19
		libmio_mborder	
		
fsys_savgps_01:	rcall	fsys_select		;select a file
		brtc	fsys_savgps_04		;ESC		
		lds	tempreg1,libmio_fileno
		sts	partab,tempreg1
		rcall	fsys_gettype		;get type
		cpi	tempreg1,0xe0		;empty
		brcs	fsys_savgps_01		;not

fsys_savgps_02:	rcall	fsys_free		;count free pages
		cpi	YH,0x00			;<256
		brne	fsys_savgps_03
		cpi	YL,0x05			;needed pages for EEPROM
		brcc	fsys_savgps_03		;we have...
		rjmp	fsys_savgps_01		;no->loop
fsys_savgps_03:	rcall	fsys_eesave		;save this file
		set				;done
		ret
		
fsys_savgps_04:	ret				;end

;------------------------------------------------------------------------------
;save file (2K), ram data
;------------------------------------------------------------------------------
fsys_savemd:	libdfl_size			;check for DF
		cpi	tempreg1,0		;none
		breq	fsys_savgps_04		;exit if no DF available
		clr	tempreg4		;first file	
fsys_savemd_00:	rcall	fsys_check		;check for dataflash
		ldi	ZL,LOW(text_fsys_10*2)	;text
		ldi	ZH,HIGH(text_fsys_10*2)
		ldi	XL,8
		libmio_mborder	

fsys_savemd_01:	rcall	fsys_select		;select a file
		brtc	fsys_savemd_06		;ESC		
		lds	tempreg1,libmio_fileno
		sts	partab,tempreg1
		rcall	fsys_gettype		;get type
		cpi	tempreg1,0xe0		;empty
		brcs	fsys_savemd_01		;not

fsys_savemd_02:	rcall	fsys_free		;count free pages
		cpi	YH,0x00			;<256
		brne	fsys_savemd_03
		cpi	YL,0x08			;needed pages for data
		brcc	fsys_savemd_03		;we have...
		rjmp	fsys_savemd_01		;no->loop
fsys_savemd_03:	ldi	ZL,LOW(fsys_ndtext*2)
		ldi	ZH,HIGH(fsys_ndtext*2)
		ldi	tempreg2,12
		libmio_newtext
		brtc	fsys_savemd_06
		ret

		rcall	fsys_rsave		;save this file
		set				;done
fsys_savemd_06:	ret
		

;------------------------------------------------------------------------------
;load file (3K)
;------------------------------------------------------------------------------
fsys_load3k:	libdfl_size			;check for DF
		cpi	tempreg1,0		;none
		breq	fsys_load3k_03		;exit if no DF available
		clr	tempreg4		;first file	
fsys_load3k_00:	rcall	fsys_check		;check for dataflash
		ldi	ZL,LOW(text_fsys_5*2)	;text pointer
		ldi	ZH,HIGH(text_fsys_5*2)
		ldi	XL,1		
		libmio_mborder		

		lds	tempreg1,libmio_lastmenu
		subi	tempreg1,0xcf
		libmio_outchar
		ldi	ZL,LOW(text_fsys_7*2)	;text pointer
		ldi	ZH,HIGH(text_fsys_7*2)
		libmio_romtext				

				
fsys_load3k_01:	rcall	fsys_select		;select a file
		brtc	fsys_load3k_03		;ESC		
		lds	tempreg1,libmio_fileno
		sts	partab,tempreg1
		rcall	fsys_gettype		;get type
		cpi	tempreg1,0x10		;BASIC
		breq	fsys_load3k_02
		cpi	tempreg1,0x16		;CH8
		breq	fsys_load3k_02
		cpi	tempreg1,0x18		;NAT
		brne	fsys_load3k_01
fsys_load3k_02:	rcall	fsys_bload		;load this file
		set				;done
fsys_load3k_03:	ret				;end		

;------------------------------------------------------------------------------
;load file (1,25K) to gps
;------------------------------------------------------------------------------
fsys_loadgps:	libdfl_size			;check for DF
		cpi	tempreg1,0		;none
		breq	fsys_ldgps_03		;exit if no DF available
		clr	tempreg4		;first file	
fsys_ldgps_00:	rcall	fsys_check		;check for dataflash
		ldi	ZL,LOW(text_fsys_14*2)	;text
		ldi	ZH,HIGH(text_fsys_15*2)
		ldi	XL,28
		libmio_mborder	
				
fsys_ldgps_01:	rcall	fsys_select		;select a file
		brtc	fsys_ldgps_03		;ESC		
		lds	tempreg1,libmio_fileno
		sts	partab,tempreg1
		rcall	fsys_gettype		;get type
		cpi	tempreg1,0x1a		;EEP
		brne	fsys_ldgps_01
fsys_ldgps_02:	ldi	tempreg2,0xff		;load all
		rcall	fsys_eeload		;load this file
		set				;done
fsys_ldgps_03:	ret				;end		

;------------------------------------------------------------------------------
;load file (2K) to meas
;------------------------------------------------------------------------------
fsys_loadmd:	libdfl_size			;check for DF
		cpi	tempreg1,0		;none
		breq	fsys_ldmd_03		;exit if no DF available
		clr	tempreg4		;first file	
fsys_ldmd_00:	rcall	fsys_check		;check for dataflash
		ldi	ZL,LOW(text_fsys_15*2)	;text
		ldi	ZH,HIGH(text_fsys_15*2)
		ldi	XL,25
		libmio_mborder		
				
fsys_ldmd_01:	rcall	fsys_select		;select a file
		brtc	fsys_ldmd_03		;ESC		
		lds	tempreg1,libmio_fileno
		sts	partab,tempreg1
		rcall	fsys_gettype		;get type
		cpi	tempreg1,0x1c		;RAMData
		brne	fsys_ldmd_01
fsys_ldmd_02:	rcall	fsys_rload		;load this file
		set				;done
fsys_ldmd_03:	ret				;end		

;------------------------------------------------------------------------------
;select an entry
;------------------------------------------------------------------------------
fsys_select:	ldi	tempreg4,0		;start entry as first
		clr	tempreg2		;entry to select
				
fsys_sel_01:	rcall	fsys_list		;list files
fsys_sel_02:	push	tempreg4
		clt
		ldi	r19,5
		libmio_menu5			;draw list
		pop	tempreg4
		cpi	tempreg1,0xed		;ESC
		brne	fsys_sel_10		;no
		clt				;clear valid flag
		ret				;end now
		
fsys_sel_10:	cpi	tempreg1,0xea		;enter
		brne	fsys_sel_15		;no
		mov	tempreg1,tempreg4	;page offset
		add	tempreg1,tempreg2	;file in page
		sts	libmio_fileno,tempreg1
		ldi	tempreg1,0xea		;restore enter
		set				;set valid flag
		ret				;end now
						
fsys_sel_15:	cpi	tempreg1,0xe2		;left
		brne	fsys_sel_30
		cpi	tempreg4,0x05
		brcs	fsys_sel_02		;do nothing
		subi	tempreg4,0x05		;-5
		rjmp	fsys_sel_01		;page change

fsys_sel_30:	cpi	tempreg1,0xe3		;right
		brne	fsys_sel_02		;no
		cpi	tempreg4,0xfc
		brcc	fsys_sel_02		;do nothing		
		subi	tempreg4,0xfb		;+5
		rjmp	fsys_sel_01		;page change		
		
;------------------------------------------------------------------------------
;check flash for useability
;------------------------------------------------------------------------------
fsys_check:	call	fsys_maxpage		;get flash status
		cpi	tempreg1,0x00		;no
		brne	fsys_check_01
		pop	tempreg1		;kill stack
		pop	tempreg1		;kill stack
		clt				;clear valid flag
		ret

fsys_check_01:	sts	partab+8,tempreg1	;store no of pages
fsys_check_02:	libmio_sync
		libmio_sync
		rcall	fsys_checkf		;test for valid format
		cpi	tempreg1,0x01		;OK
		breq	fsys_check_04
		pop	tempreg1		;kill stack
		pop	tempreg1		;killstack
		clt				;clear valid flag
		ret

fsys_check_04:	ret
		
;------------------------------------------------------------------------------
;list 5 files  
;tempreg4=first file
;------------------------------------------------------------------------------
fsys_list:	push	ZL
		push	ZH
		push	YL
		push	YH
		push	XL
		push	XH
		push	tempreg1
		push	tempreg2
		push	tempreg3
		push	tempreg4
		libmio_nokey
		ldi	YL,LOW(libmio_mram+20)		;mem array, ignore head
		ldi	YH,HIGH(libmio_mram+20)		
		
fsys_list_01:	ldi	tempreg3,5			;number of entries to view

;output file number
fsys_list_02:	ldi	tempreg1,0x30			;0
		mov	tempreg2,tempreg4		;number
fsys_list_03:	cpi	tempreg2,100
		brcs	fsys_list_04			;<100
		inc	tempreg1
		subi	tempreg2,100
		rjmp	fsys_list_03
		
fsys_list_04:	st	Y+,tempreg1			;fileno *100
		ldi	tempreg1,0x30			;0
fsys_list_05:	cpi	tempreg2,10
		brcs	fsys_list_06			;<100
		inc	tempreg1
		subi	tempreg2,10
		rjmp	fsys_list_05
		
fsys_list_06:	st	Y+,tempreg1			;fileno *100					
		subi	tempreg2,0xd0			;+0x30
		st	Y+,tempreg2

		
;get file type and write
		ldi	tempreg2,'['
		st	Y+,tempreg2	
		sts	partab,tempreg4
		rcall	fsys_gettype		;get file type to tempreg1
		rcall	fsys_pft		;write file type
		ldi	tempreg2,']'
		st	Y+,tempreg2		
		cpi	tempreg1,0xff		;empty
		brne	fsys_list_19		;no (brne)
				
		ldi	tempreg2,0x0c		;name length 
		ldi	tempreg1,0x20		;spaces
fsys_list_10:	st	Y+,tempreg1		;write space char
		dec	tempreg2		;loop counter
		brne	fsys_list_10		;loop
		rjmp	fsys_list_17		;ok, outer loop
				

;get filename and write
fsys_list_19:	sts	partab,tempreg4		;set file number
		ldi	tempreg1,0x00		;debug
		sts	partab+2,tempreg1
		ldi	XL,LOW(bas_array+512)	;no copy
		ldi	XH,HIGH(bas_array+512)		
		sts	partab+4,XL
		sts	partab+5,XH
		clr	ereg
		rcall	fsys_read		;read this page
		ldi	XL,LOW(bas_array+512)	;no copy
		ldi	XH,HIGH(bas_array+512)		
		
		ldi	tempreg2,0x0c		;name length
fsys_list_14:	ld	tempreg1,X+		;get char
		cpi	tempreg1,0x80		;valid?
		brcs	fsys_list_15		;branch if yes
		ldi	tempreg1,'.'		;replace
fsys_list_15:	cpi	tempreg1,0x20		;valid?
		brcc	fsys_list_16		;branch if yes
		ldi	tempreg1,' '		;replace
fsys_list_16:	st	Y+,tempreg1
		dec	tempreg2
		brne	fsys_list_14		;loop

		
		
fsys_list_17:	inc	tempreg4		;file+1
		dec	tempreg3		;counter -1
		breq	fsys_list_18
		rjmp	fsys_list_02
		
fsys_list_18:	pop	tempreg4
		pop	tempreg3
		pop	tempreg2
		pop	tempreg1
		pop	XH
		pop	XL
		pop	YH
		pop	YL
		pop	ZH
		pop	ZL
		ret

;------------------------------------------------------------------------------
;save flash->DF
;------------------------------------------------------------------------------
fsys_bsave:	lds	tempreg1,libmio_fileno	;file number
		sts	partab,tempreg1		;file number
	    	ldi	tempreg1,0x0c		;12 pages
		sts	partab+2,tempreg1	;set
		lds	ctrl,libmio_lastmenu	;program number
		ldi	ZL,LOW(bas_programs*2)
		ldi	ZH,HIGH(bas_programs*2)
		andi	ctrl,0x07		;limit programs
		swap	ctrl			;fileno*16
		ldi	tempreg1,0xc0		;*192
		mul	ctrl,tempreg1
		add	ZL,r0			;file offset
		adc	ZH,r1
		adiw	ZL,12			;skip filename
		lpm	XL,Z			;get type char	
		ldi	tempreg1,0x10		;file type=BAS
		cpi	XL,'C'
		brne	fsys_bsave_1
		ldi	tempreg1,0x16		;file Type=CH8
fsys_bsave_1:	cpi	XL,'N'
		brne	fsys_bsave_2
		ldi	tempreg1,0x18		;file type=NAT
fsys_bsave_2:	sts	partab+4,tempreg1
		rcall	fsys_create		;create new file
		
fsys_bover:	lds	tempreg1,libmio_fileno
		sts	partab,tempreg1		;file number
		lds	ctrl,libmio_lastmenu	;program number
		ldi	ZL,LOW(bas_programs*2)
		ldi	ZH,HIGH(bas_programs*2)
		andi	ctrl,0x07		;limit programs
		swap	ctrl			;fileno*16
		ldi	tempreg1,0xc0		;*192
		mul	ctrl,tempreg1
		add	ZL,r0			;file offset
		adc	ZH,r1
		ldi	tempreg1,0x00		;first data page
fsys_bover_1:	push	tempreg1
		ldi	r16,0x00		;bytes to copy
		ldi	YL,LOW(bas_array+512)	;buffer
		ldi	YH,HIGH(bas_array+512)
		sts	partab+4,YL
		sts	partab+5,YH
fsys_bover_2:	lpm	r17,Z+			;get flash data
		st	Y+,r17			;write to buffer			
		dec	r16
		brne	fsys_bover_2
		sts	partab+2,tempreg1	;page number
		rcall	fsys_write		;write page
		pop	tempreg1
		inc	tempreg1
		cpi	tempreg1,0x0c		;all done
		brne	fsys_bover_1
		ret

;------------------------------------------------------------------------------
;save RAM->DF
;------------------------------------------------------------------------------
fsys_rsave:	lds	tempreg1,libmio_fileno	;file number
		sts	partab,tempreg1		;file number
	    	ldi	tempreg1,0x08		;8 pages
		sts	partab+2,tempreg1	;set
		
		ldi	tempreg1,0x1c		;file Type=RAM
fsys_rsave_1:	sts	partab+4,tempreg1
		rcall	fsys_create		;create new file
		
fsys_rover:	lds	tempreg1,libmio_fileno
		sts	partab,tempreg1		;file number
		ldi	YL,LOW(rdata_buf)	;buffer
		ldi	YH,HIGH(rdata_buf)
		sts	partab+4,YL
		sts	partab+5,YH
		ldi	tempreg1,0x00		;first data page
fsys_rover_1:	push	tempreg1
		sts	partab+2,tempreg1	;page number
		rcall	fsys_write		;write page
		lds	YH,partab+5		;mext part in RAM
		inc	YH
		sts	partab+5,YH
		pop	tempreg1
		inc	tempreg1
		cpi	tempreg1,0x0c		;all done
		brne	fsys_rover_1
		ret

;------------------------------------------------------------------------------
;save EEPROM->DF
;------------------------------------------------------------------------------
fsys_eesave:	
fsys_eesave_0:	lds	tempreg1,libmio_fileno	;file number
		sts	partab,tempreg1		;file number
	    	ldi	tempreg1,0x05		;5 pages
		sts	partab+2,tempreg1	;set
		ldi	tempreg1,0x1a		;file type=LOG
fsys_eesave_1:	sts	partab+4,tempreg1
		rcall	fsys_create		;create new file
		rcall	fsys_stext
		clr	YL			;start EE
		clr	YH

fsys_eesave_5:	ldi	r16,0			;full page
		ldi	ZL,LOW(bas_array+512)	;buffer
		ldi	ZH,HIGH(bas_array+512)
		sts	partab+4,ZL
		sts	partab+5,ZH
fsys_eesave_6:	libeep_read
		st	Z+,tempreg1
		dec	r16
		brne	fsys_eesave_6
		lds	tempreg1,partab+2
		inc	tempreg1
		sts	partab+2,tempreg1	;page number
		rcall	fsys_write		;write page
		lds	tempreg1,partab+2
		cpi	tempreg1,0x05		;last page is written
		brne	fsys_eesave_5		;no						
fsys_eesave_en:	ret

;------------------------------------------------------------------------------
;load basic
;------------------------------------------------------------------------------
fsys_bload:	lds	ctrl,libmio_lastmenu
		andi	ctrl,0x07		;8 programs
		ldi	ZL,LOW(bas_programs*2)
		ldi	ZH,HIGH(bas_programs*2)
		swap	ctrl			;fileno*16
		ldi	tempreg1,0xc0		;*192
		mul	ctrl,tempreg1
		add	ZL,r0			;file offset
		adc	ZH,r1
		ldi	tempreg1,0x00		;first data page
fsys_bload1:	push	tempreg1
		sts	partab+2,tempreg1	;page number
		lds	tempreg1,libmio_fileno	;file number
		sts	partab,tempreg1		;file number

		ldi	YL,LOW(bas_array+512)	;buffer
		ldi	YH,HIGH(bas_array+512)	
		sts	partab+4,YL
		sts	partab+5,YH

		rcall	fsys_read
		ldi	YL,LOW(bas_array+512)	;buffer
		ldi	YH,HIGH(bas_array+512)	
		call	mem_writepage		;write to flash
		adiw	ZL,2			;next flash page
		pop	tempreg1
		inc	tempreg1		;page counter
		cpi	tempreg1,0x0c		;last?
		brne	fsys_bload1		;no
		sei
		ret	

;------------------------------------------------------------------------------
;load RAM
;------------------------------------------------------------------------------
fsys_rload:	rcall	fsys_ltext
		ldi	tempreg1,0x00		;first data page
fsys_rload_1:	sts	partab+2,tempreg1	;page number
		ldi	YL,LOW(rdata_buf)	;buffer
		ldi	YH,HIGH(rdata_buf)	
		sts	partab+4,YL
		sts	partab+5,YH
		lds	tempreg1,libmio_fileno
		sts	partab,tempreg1		;file number
		rcall	fsys_read
		lds	YH,partab+5		;mext part in RAM
		inc	YH
		sts	partab+5,YH
		lds	tempreg1,partab+2	;get page number
		inc	tempreg1
		cpi	tempreg1,7		;end?
		brne	fsys_rload_1		;no
		ret		

;------------------------------------------------------------------------------
;load EEPROM
;------------------------------------------------------------------------------
fsys_eeload:	rcall	fsys_ltext
		ldi	tempreg1,0x00		;first data page
		clr	YL			;start EEPROM adr
		clr	YH		
fsys_eeload_1:	sts	partab+2,tempreg1	;page number
		ldi	ZL,LOW(bas_array+512)	;buffer
		ldi	ZH,HIGH(bas_array+512)	
		sts	partab+4,ZL
		sts	partab+5,ZH
		lds	tempreg1,libmio_fileno
		sts	partab,tempreg1		;file number
		rcall	fsys_read
		ldi	ZL,LOW(bas_array+512)	;buffer
		ldi	ZH,HIGH(bas_array+512)
		ldi	r16,0
fsys_eeload_2:	ld	tempreg1,Z+		;get byte from buffer
		libeep_write			;put in EEPROM
		dec	r16
		brne	fsys_eeload_2
		lds	tempreg1,partab+2	;get page number
		inc	tempreg1
		cpi	tempreg1,5		;end?
		brne	fsys_eeload_1		;no
		ret		
		

;------------------------------------------------------------------------------
;print file type (tempreg1)
;------------------------------------------------------------------------------
fsys_pft:	cpi	tempreg1,0x10		;BASIC
		brne	fsys_pft_01		;no
		ldi	tempreg2,'B'
		st	Y+,tempreg2
		ret

fsys_pft_01:	cpi	tempreg1,0x12		;BACKUP
		brne	fsys_pft_02		;no
		ldi	tempreg2,'M'
		st	Y+,tempreg2
		ret

fsys_pft_02:	cpi	tempreg1,0x14		;USER
		brne	fsys_pft_03		;no
		ldi	tempreg2,'U'
		st	Y+,tempreg2
		ret

fsys_pft_03:	cpi	tempreg1,0x16		;CH8
		brne	fsys_pft_04		;no
		ldi	tempreg2,'C'
		st	Y+,tempreg2
		ret		
		
fsys_pft_04:	cpi	tempreg1,0x18		;NAT
		brne	fsys_pft_05		;no
		ldi	tempreg2,'N'
		st	Y+,tempreg2
		ret				
		
fsys_pft_05:	cpi	tempreg1,0x1a		;GPS data
		brne	fsys_pft_06		;no
		ldi	tempreg2,'G'
		st	Y+,tempreg2
		ret			
	
fsys_pft_06:	cpi	tempreg1,0x1c		;Sample data
		brne	fsys_pft_07		;no
		ldi	tempreg2,'S'
		st	Y+,tempreg2
		ret			

fsys_pft_07:	cpi	tempreg1,0x1e		;thermo data
		brne	fsys_pft_99		;no
		ldi	tempreg2,'T'
		st	Y+,tempreg2
		ret			
		
fsys_pft_99:	ldi	tempreg1,0xff
		ldi	tempreg2,'-'
		st	Y+,tempreg2
		ret			
		
;------------------------------------------------------------------------------
;text output routines
;------------------------------------------------------------------------------
fsys_ltext:	libmio_mborder
		ldi	ZL,LOW(text_fsys_1*2)	;text load
		ldi	ZH,HIGH(text_fsys_1*2)
fsys_ltext_1:	ldi	XL,10
		ldi	XH,20		
		libmio_xztext			
		ret

fsys_stext:	libmio_mborder
		ldi	ZL,LOW(text_fsys_2*2)	;text save
		ldi	ZH,HIGH(text_fsys_2*2)
		ldi	XL,10
		ldi	XH,20		
		libmio_xztext			
		ret

										
;------------------------------------------------------------------------------
;subroutines
;------------------------------------------------------------------------------
fsys_free:
		libdfl_free
		ret
		
fsys_gettype:
		libdfl_gettype
		ret
		
fsys_size:
		libdfl_size
		ret
		
fsys_gword:
		libdfl_gword
		ret
		
fsys_checkf:
		libdfl_checkformat
		ret
		
fsys_create:
		libdfl_create
		ret
		
fsys_delete:
		libdfl_delete
		ret
		
fsys_read:
		libdfl_read
		ret
		
fsys_write:
		libdfl_write
		ret
		
fsys_maxpage:
		libdfl_maxpage
		ret
		

				