;################################################################################
;#										#
;# avr-chipbasic2M - mobile single chip basic computer with ATmega644		#
;# X-Modem routines								#
;# copyright (c) 2008 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 2		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################		
		
;-----------------------------------------------------------------------------
; send program to XMODEM
;-----------------------------------------------------------------------------
send_prog:	ldi	tempreg3,1			;first block number
		rcall   xm_gserb			;get char
		
		lds	tempreg2,libmio_keycode		;get keycode
		cpi	tempreg2,0xed			;ESC
		brne	send_prog_01			;not
send_prog_00:	ret					;OK, done
				
send_prog_01:	cpi	tempreg1,0x15			;NAK?
		brne	send_prog			;no->loop
		
		lds	ctrl,libmio_lastmenu		;program number
		ldi	ZL,LOW(bas_programs*2)
		ldi	ZH,HIGH(bas_programs*2)
		andi	ctrl,0x07			;limit programs
		swap	ctrl				;progno*16
		ldi	tempreg1,0xc0			;*192
		mul	ctrl,tempreg1
		add	ZL,r0				;add offset
		adc	ZH,r1
		
send_prog_02:	ldi	YL,LOW(bas_array)
		ldi	YH,HIGH(bas_array)
		ldi	tempreg2,0
send_prog_03:	lpm	tempreg1,Z+			;copy to RAM
		st	Y+,tempreg1				
		dec	tempreg2
		brne	send_prog_03	
		
send_prog_04:	ldi	YL,LOW(bas_array)
		ldi	YH,HIGH(bas_array)
		rcall	xm_txbl				;send data block		
		rcall   xm_gserb			;get char		
		lds	tempreg2,libmio_keycode		;get keycode
		cpi	tempreg2,0xed			;ESC
		breq	send_prog_00			;yes
		cpi	tempreg1,0x06			;ACK?
		brne	send_prog_04			;no->repeat
		
		inc	tempreg3			;block+1

send_prog_05:	ldi	YL,LOW(bas_array+128)
		ldi	YH,HIGH(bas_array+128)
		rcall	xm_txbl				;send data block		
		rcall   xm_gserb			;get char		
		lds	tempreg2,libmio_keycode		;get keycode
		cpi	tempreg2,0xed			;ESC
		breq	send_prog_00			;yes
		cpi	tempreg1,0x06			;ACK?
		brne	send_prog_05			;no->repeat				
		
		inc	tempreg3
		cpi	tempreg3,25			;all done
		brne	send_prog_02			;no
		ldi	tempreg1,0x04			;send EOT
		rcall   xm_pser						
		ret					;yes


;-----------------------------------------------------------------------------
; send data to XMODEM
;-----------------------------------------------------------------------------
send_ram:	ldi	tempreg3,1			;first block number
		rcall   xm_gserb				;get char
		
		lds	tempreg2,libmio_keycode		;get keycode
		cpi	tempreg2,0xed			;ESC
		brne	send_ram_01			;not
send_ram_00:	ret					;OK, done
				
send_ram_01:	cpi	tempreg1,0x15			;NAK?
		brne	send_ram			;no->loop

		ldi	YL,LOW(rdata_head)
		ldi	YH,HIGH(rdata_head)
		
send_ram_04:	rcall	xm_txbl				;send data block		
		rcall   xm_gserb			;get char		
		lds	tempreg2,libmio_keycode		;get keycode
		cpi	tempreg2,0xed			;ESC
		breq	send_ram_00			;yes
		cpi	tempreg1,0x06			;ACK?
		brne	send_ram_05			;no->repeat		
		inc	tempreg3			;block+1
		cpi	tempreg3,17			;all done
		brne	send_ram_04			;no
		ldi	tempreg1,0x04			;send EOT
		rcall   xm_pser						
		ret					;yes

send_ram_05:	sbiw	YL,63				;reset ptr
		sbiw	YL,62
		sbiw	YL,2
		rjmp	send_ram_04

;-----------------------------------------------------------------------------
; receive data from XMODEM
;-----------------------------------------------------------------------------
recv_ram:	ldi	tempreg1,0x15			;send NAK to start
		rcall   xm_pser
		ldi	tempreg3,0			;blocks received		
		ldi	YL,LOW(rdata_head)		;address
		ldi	YH,HIGH(rdata_head)

recv_ram_01:	push	tempreg3
		rcall	xm_rxbl				;send data block
		cpi	ereg,0
		breq	recv_ram_01a
		cpi	ereg,1
		breq	recv_ram_end
		cpi	ereg,40
		breq	recv_ram_end			;abort condition
		ldi	tempreg1,0x15			;send NAK
		rcall   xm_pser		
		rjmp	recv_ram_01			;receive again
recv_ram_01a:	ldi	tempreg1,0x06			;send ACK
		rcall   xm_pser	

		pop	tempreg3
		inc	tempreg3
		cpi	tempreg3,16
		brne	recv_ram_01

recv_ram_04:	rcall   xm_gser
		cpi	tempreg1,0x04			;EOT
		brne	recv_ram_04
		ldi	tempreg1,0x06			;send ACK
		rcall   xm_pser	
		ret
		
		
recv_ram_end:	pop	tempreg3
		ret		

				
;------------------------------------------------------------------------------
;receive program via XMODEM
;------------------------------------------------------------------------------
recv_prog:	lds	ctrl,libmio_lastmenu
		andi	ctrl,0x07			;8 programs
		ldi	ZL,LOW(bas_programs*2)
		ldi	ZH,HIGH(bas_programs*2)
		swap	ctrl				;fileno*16
		ldi	tempreg1,0xc0			;*192
		mul	ctrl,tempreg1
		add	ZL,r0				;file offset
		adc	ZH,r1
		
		ldi	tempreg1,0x15			;send NAK to start
		rcall   xm_pser
		ldi	tempreg3,0			;blocks received		
recv_prog_00:	push	tempreg3

		
recv_prog_01:	ldi	YL,LOW(bas_array+512)
		ldi	YH,HIGH(bas_array+512)
		rcall	xm_rxbl				;receive data block
		cpi	ereg,0
		breq	recv_prog_01a
		cpi	ereg,1
		breq	recv_prog_end
		cpi	ereg,40
		breq	recv_prog_end			;abort condition
		ldi	tempreg1,0x15			;send NAK
		rcall   xm_pser		
		rjmp	recv_prog_01			;receive again
recv_prog_01a:	ldi	tempreg1,0x06			;send ACK
		rcall   xm_pser	
		
recv_prog_02:	ldi	YL,LOW(bas_array+640)
		ldi	YH,HIGH(bas_array+640)
		rcall	xm_rxbl				;receive data block			
		cpi	ereg,0
		breq	recv_prog_03
		cpi	ereg,1
		breq	recv_prog_end
		cpi	ereg,40
		breq	recv_prog_end			;abort condition
		ldi	tempreg1,0x15			;send NAK
		rcall   xm_pser		
		rjmp	recv_prog_02			;receive again		
		
recv_prog_03:	call	mem_writepage			;write to flash
		sei
		adiw	ZL,2				;next flash page
		ldi	tempreg1,0x06			;send ACK
		rcall   xm_pser		
		pop	tempreg3
		inc	tempreg3
		cpi	tempreg3,12
		brne	recv_prog_00

recv_prog_04:	rcall   xm_gser
		cpi	tempreg1,0x04			;EOT
		brne	recv_prog_04
		ldi	tempreg1,0x06			;send ACK
		rcall   xm_pser	
		ret
		
recv_prog_end:	pop	tempreg3
		ret		
		

;-----------------------------------------------------------------------------
; send internal EEPROM (1792 bytes log area) to XMODEM
;-----------------------------------------------------------------------------
send_intee:	ldi	tempreg3,1			;first block number
		rcall   xm_gserb				;get char
		
		lds	tempreg2,libmio_keycode		;get keycode
		cpi	tempreg2,0xed			;ESC
		brne	send_intee_01			;not
send_intee_00:	ret					;OK, done
				
send_intee_01:	cpi	tempreg1,0x15			;NAK?
		brne	send_intee			;no->loop
		
send_intee_02:	ldi	XL,128				;bytes per page
		mov	tempreg4,tempreg3
		dec	tempreg4
		mul	XL,tempreg4
		movw	YL,r0				;get EE start address
		ldi	XL,LOW(bas_array)
		ldi	XH,HIGH(bas_array)
		ldi	tempreg4,128			;number of bytes
send_intee_03:	libeep_read				;copy to RAM
		st	X+,tempreg1				
		dec	tempreg4
		brne	send_intee_03
				
send_intee_04:	ldi	YL,LOW(bas_array)
		ldi	YH,HIGH(bas_array)
		rcall	xm_txbl				;send data block		
		rcall   xm_gserb				;get char		
		lds	tempreg2,libmio_keycode		;get keycode
		cpi	tempreg2,0xed			;ESC
		breq	send_intee_00			;yes
		cpi	tempreg1,0x06			;ACK?
		brne	send_intee_04			;no->repeat
		inc	tempreg3			;block+1
		cpi	tempreg3,15			;all done
		brne	send_intee_02			;no
		ldi	tempreg1,0x04			;send EOT
		rcall   xm_pser								
		ret					;yes
		
;------------------------------------------------------------------------------
;receive internal EEPROM via XMODEM (1792 bytes log area)
;------------------------------------------------------------------------------
recv_intee:	ldi	tempreg1,0x15			;send NAK to start
		rcall    xm_pser
		ldi	tempreg3,0			;blocks received		
recv_intee_00:	push	tempreg3

recv_intee_01:	ldi	YL,LOW(bas_array)
		ldi	YH,HIGH(bas_array)
		rcall	xm_rxbl
		cpi	ereg,0
		breq	recv_intee_02
		cpi	ereg,40
		breq	recv_intee_end			;abort condition
		ldi	tempreg1,0x15			;send NAK
		rcall   xm_pser		
		rjmp	recv_intee_01			;receive again
recv_intee_02:	
		ldi	XL,128				;bytes per page
		mov	tempreg4,tempreg3
		dec	tempreg4
		mul	XL,tempreg4
		movw	YL,r0				;get EE start address
		ldi	XL,LOW(bas_array)
		ldi	XH,HIGH(bas_array)
recv_intee_03:	ldi	tempreg4,128			;number of bytes
recv_intee_04:	ld	tempreg1,X+
		libeep_write				
		dec	tempreg4
		brne	recv_intee_04
		
		ldi	tempreg1,0x06			;send ACK
		rcall   xm_pser			
		
		pop	tempreg3
		inc	tempreg3
		cpi	tempreg3,14
		brne	recv_intee_00
recv_intee_05:	rcall   xm_gser
		cpi	tempreg1,0x04			;EOT
		brne	recv_intee_05
		ldi	tempreg1,0x06			;send ACK
		rcall	xm_pser				
		ret
		
recv_intee_end:	pop	tempreg3
		ret		
				
	
;------------------------------------------------------------------------------
;put tempreg1 to serial output
;------------------------------------------------------------------------------
xm_pser:	push	tempreg1
xm_pser1:	lds	tempreg1,UCSR0A
		sbrs	tempreg1,UDRE0
		rjmp	xm_pser1
		pop	tempreg1
		sts	UDR0,tempreg1
		ret					;thats all

;------------------------------------------------------------------------------
;get tempreg1 from serial input
;------------------------------------------------------------------------------
xm_gser:	lds	tempreg1,UCSR0A
		sbrs	tempreg1,RXC0
		rjmp	xm_gser
		lds	tempreg1,UDR0
		ret					;thats all

;------------------------------------------------------------------------------
;get tempreg1 from serial input with break
;------------------------------------------------------------------------------
xm_gserb:	lds	tempreg1,UCSR0A
		sbrc	tempreg1,RXC0
		rjmp	xm_gserb_3
xm_gserb_2:	lds	tempreg1,libmio_keycode		;get keycode
		cpi	tempreg1,0x00			;key pressed?
		breq	xm_gserb			;no key is pressed
		ldi	tempreg1,0x1b			;break code
		ret					;thats all
			
xm_gserb_3:	lds	tempreg1,UDR0
		ret					;thats all

				
;-----------------------------------------------------------------------------
; read XMODEM block, tempreg3=blocknum, Y=address
;-----------------------------------------------------------------------------
xm_rxbl:	push	tempreg1
		push	tempreg2
		push	tempreg4
xm_rxbl_lo:	rcall	xm_gserb			;get char
		cpi	tempreg1,0x1b			;ESC
		brne	xm_rxbl_0			;no
		ldi	ereg,1				;break
		rjmp	xm_rxbl_03			;end
		
xm_rxbl_0:	cpi	tempreg1,0x01			;SOH
		brne	xm_rxbl_lo			;end with error
		rcall	xm_gser				;next char=block num
		mov	tempreg3,tempreg1
		rcall	xm_gser				;next char=inv block num
		com	tempreg1
		cp	tempreg3,tempreg1		
		breq	xm_rxbl_01			;OK
xm_rxbl_00:	ldi	ereg,40				;error
		rjmp	xm_rxbl_03			;end
		
xm_rxbl_01:	ldi	tempreg4,128			;num of bytes
		clr	tempreg2			;checksum
xm_rxbl_02:	rcall	xm_gser				;get data byte
		add	tempreg2,tempreg1
		st	Y+,tempreg1
		dec	tempreg4
		brne	xm_rxbl_02
		rcall	xm_gser				;get checksum byte				
		cp	tempreg1,tempreg2		;OK
		brne	xm_rxbl_04
		clr	ereg				;OK
xm_rxbl_03:	pop	tempreg4
		pop	tempreg2
		pop	tempreg1
		ret
		
xm_rxbl_04:	ldi	ereg,41				;checksum error
		rjmp	xm_rxbl_03		
		
;-----------------------------------------------------------------------------
; write XMODEM block, tempreg3=blocknum, Y=address
;-----------------------------------------------------------------------------
xm_txbl:	push	tempreg1
		push	tempreg2
		push	tempreg4
		ldi	tempreg1,0x01			;SOH
		rcall	xm_pser				;out
		mov	tempreg1,tempreg3
		rcall	xm_pser				;blocknum
		com	tempreg1
		rcall	xm_pser				;inv. blocknum
		
xm_txbl_01:	ldi	tempreg4,128			;num of bytes
		clr	tempreg2			;checksum
xm_txbl_02:	ld	tempreg1,Y+
		add	tempreg2,tempreg1
		rcall	xm_pser				;get data byte
		dec	tempreg4
		brne	xm_txbl_02
		mov	tempreg1,tempreg2		;copy checksum
		rcall	xm_pser				;out checksum byte				
xm_txbl_03:	pop	tempreg4
		pop	tempreg2
		pop	tempreg1
		ret				
				
		