;################################################################################
;#										#
;# avr-chipbasic2M - mobile single chip basic computer with ATmega644		#
;# puzzle game									#
;# copyright (c) 2008 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 2		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;------------------------------------------------------------------------------
;main screen mask
;------------------------------------------------------------------------------
puzz_main:	ldi	ZL,LOW(libmio_icons*2+450)
		ldi	ZH,HIGH(libmio_icons*2+450)
		libmio_appframe
		sts	libmio_color,const_0		;invers
		ldi	YL,11
		ldi	YH,10
		ldi	ZL,116
		ldi	ZH,62
		libmio_fbox
		sts	libmio_color,const_1		;normal
		ldi	YL,11
		ldi	YH,10
		ldi	ZL,34
		ldi	ZH,62
		libmio_fbox
		ldi	YL,92
		ldi	YH,10
		ldi	ZL,116
		ldi	ZH,62
		libmio_fbox
		sts	bas_array+16,const_0		;puzzle to do
	
puzz_main_i:	rcall	puzz_init
		rcall	puzz_check
		brts	puzz_main_i
		mov	tempreg7,const_0	;clear moves
		mov	tempreg8,const_0
		libmio_nokey
		
puzz_main_1:	rcall	puzz_draw		;draw field
		rcall	puzz_show		;show moves
		rcall	puzz_check
		brtc	puzz_main_2		
		rjmp	puzz_main_e
		
puzz_main_2:	libmio_waitkey			;wait for key
		cpi	tempreg1,0xed		;ESC
		brne	puzz_main_3
		ret
		
puzz_main_3:	cpi	tempreg1,0xe2		;left
		brne	puzz_main_4
		rcall	puzz_ml
		brtc	puzz_main_2
		rjmp	puzz_addm		;add move
		
puzz_main_4:	cpi	tempreg1,0xe3		;right
		brne	puzz_main_5
		rcall	puzz_mr
		brtc	puzz_main_2
		rjmp	puzz_addm		;add move		
					
puzz_main_5:	cpi	tempreg1,0xe4		;up
		brne	puzz_main_6
		rcall	puzz_mu
		brtc	puzz_main_2
		rjmp	puzz_addm		;add move		

puzz_main_6:	cpi	tempreg1,0xe5		;down
		brne	puzz_main_7
		rcall	puzz_md
		brtc	puzz_main_2
		rjmp	puzz_addm		;add move		
				
puzz_main_7:	rcall	puzz_sel
		rjmp	puzz_main_i		
		
puzz_addm:	movw	XL,tempreg7
		adiw	XL,1
		movw	tempreg7,XL
		libmio_sync
		libmio_nokey				
		rjmp	puzz_main_1
		
puzz_main_e:	sts	libmio_color,const_0	;invers
		sts	libmio_font,const_0	;small font
		ldi	XL,28
		ldi	XH,27
		ldi	ZL,LOW(puzz_text_1*2)	;text pointer
		ldi	ZH,HIGH(puzz_text_1*2)
		libmio_xztext						
		
puzz_main_e1:	libmio_waitkey
		cpi	tempreg1,0xed
		brne	puzz_main_e1
		ret								

;------------------------------------------------------------------------------
;show moves
;------------------------------------------------------------------------------
puzz_show:	sts	libmio_color,const_0	;invers
		sts	libmio_font,const_1	;small font
		ldi	XL,24
		ldi	XH,2
		ldi	ZL,LOW(puzz_text_2*2)	;text pointer
		ldi	ZH,HIGH(puzz_text_2*2)
		libmio_xztext				
				
		lds	tempreg1,bas_array+16
		subi	tempreg1,0xcf
		libmio_outchar

		ldi	ZL,LOW(puzz_text_3*2)	;text pointer
		ldi	ZH,HIGH(puzz_text_3*2)
		libmio_romtext				
			
		movw	XL,tempreg7
		ldi	ctrl,0x0e
		libmio_outdez
		ret

;------------------------------------------------------------------------------
;select
;------------------------------------------------------------------------------
puzz_sel:	lds	XL,bas_array+16
		inc	XL
		cpi	XL,5
		brcs	puzz_sel_1
		clr	XL
puzz_sel_1:	sts	bas_array+16,XL
		ret				
		
;------------------------------------------------------------------------------
;init field 
;------------------------------------------------------------------------------
puzz_init:	lds	XL,bas_array+16		;which
		cpi	XL,4
		brcs	puzz_init_20		;fixed
		
		ldi	YL,LOW(bas_array)	;fill with FF
		ldi	YH,HIGH(bas_array)
		ldi	XL,0x10
		ldi	XH,0xff
puzz_init_01:	st	Y+,XH
		dec	XL
		brne	puzz_init_01
		
		ldi	YL,LOW(bas_array)	;start address
		ldi	YH,HIGH(bas_array)				
		ldi	r16,0x10		;16 values
		
puzz_init_10:	libmio_random
		eor	XL,XH
		andi	XL,0x0f
		ldi	ZL,LOW(bas_array)	;start address
		ldi	ZH,HIGH(bas_array)				
		ldi	r17,0x10
puzz_init_15:	ld	r18,Z+			;get value
		cp	r18,XL			;exists already
		breq	puzz_init_10		;try again
		dec	r17
		brne	puzz_init_15
		st	Y+,XL
		dec	r16
		brne	puzz_init_10
		ret			
		
puzz_init_20:	ldi	ZL,LOW(puzz_dat*2)
		ldi	ZH,HIGH(puzz_dat*2)
		andi	XL,3
		swap	XL
		add	ZL,XL
		adc	ZH,const_0
		ldi	YL,LOW(bas_array)	;start address
		ldi	YH,HIGH(bas_array)				
		ldi	XH,16
puzz_init_21:	lpm	XL,Z+
		st	Y+,XL
		dec	XH
		brne	puzz_init_21
		ret
								
;------------------------------------------------------------------------------
;draw field
;------------------------------------------------------------------------------
puzz_draw:	sts	libmio_font,const_0		;standard font
		sts	libmio_color,const_1		;normal	
		ldi	XL,LOW(bas_array)		;start address
		ldi	XH,HIGH(bas_array)					

		ldi	r16,0				;pos
puzz_draw_01:	mov	YL,r16
		mov	YH,r16
		ldi	ZL,14				;X-factor
		andi	YL,3
		mul	ZL,YL
		mov	YL,r0
		ldi	ZL,36				;X start
		add	YL,ZL
		
		ldi	ZL,13				;Y-factor
		andi	YH,0x0c
		lsr	YH
		lsr	YH
		mul	ZL,YH
		mov	YH,r0
		ldi	ZL,11				;Y-start
		add	YH,ZL		
;now we have pos					
		movw	ZL,YL				;copy to 2nd corner
		subi	ZL,0xf4				;X+12
		subi	ZH,0xf5				;Y+11
		ld	r18,X+				;get value of pos
		cpi	r18,0				;zero(hole)?
		breq	puzz_draw_21			;yes
		sts	libmio_color,const_1		;normal		
		libmio_fbox				;draw number box
		push	XL				;save array pos
		push	XH
		movw	XL,YL
		subi	XH,0xff				;Y+1
		cpi	r18,0x0a			;>=10
		brcc	puzz_draw_02			;
		subi	XL,0xfd				;X+2 if only 1 digit
puzz_draw_02:	libmio_gotoxy				;set position
		sts	libmio_color,const_0		;invers		
		cpi	r18,0x0a
		brcs	puzz_draw_03
		ldi	tempreg1,0x31			;'1'
		libmio_outchar
		subi	r18,0x0a
puzz_draw_03:	mov	tempreg1,r18
		subi	tempreg1,0xd0
		libmio_outchar				
		pop	XH				;restore array pos
		pop	XL		
puzz_draw_20:	inc	r16
		cpi	r16,0x10
		brne	puzz_draw_01
		sts	libmio_color,const_1		;normal		
		ret
			
puzz_draw_21:	sts	libmio_color,const_0		;invers		
		libmio_fbox
		rjmp	puzz_draw_20
		
;------------------------------------------------------------------------------
;find zero to tempreg4
;------------------------------------------------------------------------------		
puzz_findz:	ldi	XL,LOW(bas_array)		;start address
		ldi	XH,HIGH(bas_array)						
		ldi	tempreg4,0			;first position
puzz_findz_1:	ld	tempreg1,X+			;
		cpi	tempreg1,0x00			;zero?
		breq	puzz_findz_2	
		inc	tempreg4
		rjmp	puzz_findz_1
puzz_findz_2:	ret

			
;------------------------------------------------------------------------------
;check if its done (T)
;------------------------------------------------------------------------------	
puzz_check:	set					;set flag
		ldi	XL,LOW(bas_array)		;start address
		ldi	XH,HIGH(bas_array)						
		ldi	tempreg2,0			;first position
puzz_check_1:	ld	tempreg1,X+			;
		dec	tempreg1
		andi	tempreg1,0x0f
		cp	tempreg1,tempreg2
		breq	puzz_check_2
		clt					;not done (clear flag)
puzz_check_2:	inc	tempreg2
		cpi	tempreg2,0x10
		brne	puzz_check_1
		ret

;------------------------------------------------------------------------------
;move down if possible (T=1)
;------------------------------------------------------------------------------
puzz_md:	rcall	puzz_findz			;get position
		clt
		ldi	YL,LOW(bas_array)		;start address
		ldi	YH,HIGH(bas_array)
		add	YL,tempreg4
		adc	YH,const_0					
puzz_md_02:	cpi	tempreg4,4			;can we move down
		brcc	puzz_md_04
		ret					;not possible
		
puzz_md_04:	subi	tempreg4,4
		sbiw	YL,4
		ld	tempreg1,Y
		std	Y+4,tempreg1
		st	Y,const_0
		set
		ret
			
		
;------------------------------------------------------------------------------
;move up if possible (T=1)
;------------------------------------------------------------------------------
puzz_mu:	rcall	puzz_findz			;get position
		clt
		ldi	YL,LOW(bas_array)		;start address
		ldi	YH,HIGH(bas_array)
		add	YL,tempreg4
		adc	YH,const_0					
puzz_mu_02:	cpi	tempreg4,12			;can we move up
		brcs	puzz_mu_04
		ret					;not possible
		
puzz_mu_04:	subi	tempreg4,0xfc			;+4
		ldd	tempreg1,Y+4
		st	Y,tempreg1
		std	Y+4,const_0
		adiw	YL,4
		set
		ret

;------------------------------------------------------------------------------
;move left if possible (T=1)
;------------------------------------------------------------------------------
puzz_ml:	rcall	puzz_findz			;get position
		clt
		ldi	YL,LOW(bas_array)		;start address
		ldi	YH,HIGH(bas_array)
		add	YL,tempreg4
		adc	YH,const_0					
puzz_ml_02:	mov	tempreg3,tempreg4		;copy
		andi	tempreg3,0x03
		cpi	tempreg3,3			;can we move up
		brne	puzz_ml_04
		ret					;not possible
		
puzz_ml_04:	subi	tempreg4,0xff			;+1
		ldd	tempreg1,Y+1
		st	Y,tempreg1
		std	Y+1,const_0
		adiw	YL,1
		set
		ret

;------------------------------------------------------------------------------
;move right if possible (T=1)
;------------------------------------------------------------------------------
puzz_mr:	rcall	puzz_findz			;get position
		clt
		ldi	YL,LOW(bas_array)		;start address
		ldi	YH,HIGH(bas_array)
		add	YL,tempreg4
		adc	YH,const_0					
puzz_mr_02:	mov	tempreg3,tempreg4		;copy
		andi	tempreg3,0x03
		cpi	tempreg3,0			;can we move up
		brne	puzz_mr_04
		ret					;not possible
		
puzz_mr_04:	subi	tempreg4,0x01			;-1
		sbiw	YL,1
		ld	tempreg1,Y
		std	Y+1,tempreg1
		st	Y,const_0
		set
		ret
				
		
puzz_dat:	.db	3,14,12,9,6,13,10,8,0,1,15,2,7,5,4,11	;puzzle 1
		.db	10,5,11,13,7,15,12,9,2,3,14,4,8,0,1,6	;puzzle 2
		.db	2,4,9,3,6,12,8,1,7,14,0,5,11,15,13,10	;puzzle 3
		.db	11,7,15,14,13,12,8,0,1,2,4,9,3,6,5,10	;puzzle 4
															