;################################################################################
;#										#
;# avr-chipbasic2M - mobile single chip basic computer with ATmega644		#
;# serial sniffer functions							#
;# copyright (c) 2008 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 2		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;------------------------------------------------------------------------------
;main screen mask
;------------------------------------------------------------------------------
snif_main:	sts	bas_array+100,const_0		;text mode
		sts	bas_array+101,const_0		;speed
		sts	bas_array+104,const_0		;pause
		sts	bas_array+102,const_0		;byte
		sts	bas_array+103,const_0		;byte in row
		
snif_main_re:	ldi	ZL,LOW(libmio_icons*2+610)
		ldi	ZH,HIGH(libmio_icons*2+610)
		libmio_appframe
		rcall	snif_mtable			;calc mirrortable
		
snif_main_1:	rcall	snif_clrf			;clear field
snif_main_2:	rcall	snif_spar			;show parameter
		libmio_nokey				;wait for key release
		sts	libmio_color,const_1		;normal color
		sts	libmio_font,const_0		;standard font
		mov	tempreg7,const_0
		mov	tempreg8,const_0
		dec	tempreg8
		mov	r14,const_0
		mov	r15,const_0
		mov	r13,const_0
		
snif_loop:	rcall	snif_gbyte			;get byte
		rcall	snif_wdata			;write data
		
		lds	tempreg1,libmio_keycode
		
snif_loop_01:	cpi	tempreg1,0xed			;ESC
		brne	snif_loop_1
		libmio_sysconf				;restore serial setup
		ret					;END
		
snif_loop_1:	cpi	tempreg1,0xea			;ENTER (R/S)	
		brne	snif_loop_2	
		lds	XL,bas_array+104
		inc	XL
		andi	XL,1
		sts	bas_array+104,XL
		rjmp	snif_main_2

snif_loop_2:	cpi	tempreg1,0xe4			;up (clr)	
		brne	snif_loop_3	
		rcall	snif_clrf
		rjmp	snif_main_1

snif_loop_3:	cpi	tempreg1,0xe5			;down (T/H)	
		brne	snif_loop_4	
		lds	XL,bas_array+100
		inc	XL
		andi	XL,1
		sts	bas_array+100,XL
		rjmp	snif_main_1

snif_loop_4:	cpi	tempreg1,0xe2			;left (BR-)	
		brne	snif_loop_5	
		lds	XL,bas_array+101
		dec	XL
		andi	XL,7
		sts	bas_array+101,XL
		rjmp	snif_main_2

snif_loop_5:	cpi	tempreg1,0xe3			;right (BR+)	
		brne	snif_loop	
		lds	XL,bas_array+101
		inc	XL
		andi	XL,7
		sts	bas_array+101,XL
		rjmp	snif_main_2
		
				
;------------------------------------------------------------------------------
;write received byte into queue (tempreg7=write pointer)
;------------------------------------------------------------------------------
snif_gbyte:	push	tempreg1			;save registers
		push	tempreg2
		lds	tempreg1,UCSR0A			;status
		sbrc	tempreg1,RXC0			;skip if have no byte
		rjmp	snif_gbyte_2
		
snif_gbyte_1:	pop	tempreg2
		pop	tempreg1
		ret

snif_gbyte_2:	lds	tempreg1,UDR0		
		lds	tempreg2,bas_array+104		;run flag
		sbrs	tempreg2,0			;skip if run
		rjmp	snif_gbyte_1			
		
snif_gbyte_3:	cp	tempreg7,tempreg8		;can we write?
		breq	snif_gbyte_1			;no, queue is full
		push	YL				;save Y register
		push	YH
		ldi	YH,0x0e				;start buffer
		mov	YL,tempreg7		
		st	Y+,tempreg1			;store data
		mov	tempreg7,YL			;set write pointer
		pop	YH
		pop	YL
		rjmp	snif_gbyte_1

;------------------------------------------------------------------------------
;show parameter
;------------------------------------------------------------------------------
snif_spar:	sts	libmio_color,const_0
		sts	libmio_font,const_0	
		libmio_thistext		
		.db 0,12,"Baud:",0
		ldi	ZL,LOW(snif_brt*2)
		ldi	ZH,HIGH(snif_brt*2)
		lds	XL,bas_array+101
		ldi	XH,6			;chars per text
		mul	XL,XH
		add	ZL,r0
		adc	ZH,r1
		ldi	XL,6
		libmio_romtext
		lds	XL,bas_array+100
		sbrs	XL,0
		rjmp	snif_spar_1
		libmio_thistext
		.db	255," HEX ",0,0
		rjmp	snif_spar_2
snif_spar_1:	libmio_thistext
		.db	255," TXT ",0,0
snif_spar_2:	ldi	tempreg1,0xc2		;pause
		lds	XL,bas_array+104
		sbrc	XL,0
		inc	tempreg1		;->run
		libmio_outchar		
		lds	XH,bas_array+101
		swap	XH
		lsl	XH
		jmp	libmio_setser				
	
;------------------------------------------------------------------------------
;clear field
;------------------------------------------------------------------------------
snif_clrf:	ldi	YL,LOW(libmio_vram+178)		;1 base address in vram
		ldi	YH,HIGH(libmio_vram+178)	;1				
		ldi	r16,50				;lines
snif_clrf_1:	ldi	r17,12				;bytes to move				
snif_clrf_2:	st	Y+,const_0
		dec	r17
		brne	snif_clrf_2
		adiw	YL,4
		dec	r16
		brne	snif_clrf_1
		ret
		
;------------------------------------------------------------------------------
;calc mirrortable
;------------------------------------------------------------------------------		
snif_mtable:	ldi	YL,0x00				;start of table
		ldi	YH,0x0f	
snif_mtable_1:	mov	XL,YL
		ldi	XH,8
snif_mtable_2:	lsl	XL
		ror	tempreg1
		dec	XH
		brne	snif_mtable_2
		st	Y+,tempreg1
		cpi	YL,0
		brne	snif_mtable_1
		ret				
		
;------------------------------------------------------------------------------
;write data
;------------------------------------------------------------------------------		
snif_wdata:	mov	tempreg1,tempreg8		;read pointer
		inc	tempreg1			;+1
		cp	tempreg1,tempreg7
		brne	snif_wdata_01			;
snif_wdata_00:	ret					;nothing to do
		
snif_wdata_01:	lds	XL,bas_array+100
		sbrc	XL,0				;skip if text mode
		rjmp	snif_wdata_20			;goto hex mode (if available)		
;text mode		
snif_wdata_02:	ldi	YH,0x0e
		inc	tempreg8			;1 read pointer+1 
		mov	YL,tempreg8			;1
		ld	ZL,Y				;1 this is what we must write
		mov	ZH,r13				;1 old
		mov	r13,ZL				;1 this is our "new old" char

		cpi	ZL,0x0d				;CR
		brne	snif_wdata_06
		cpi	ZH,0x0a				;LF
		breq	snif_wdata_00			;ignore LF after CR
snif_wdata_04:	mov	r15,const_0			;X=0
		inc	r14				;Y+=1
		mov	ZH,r14				;copy Y
		cpi	ZH,5				;beyond end
		brcs	snif_wdata_00			;no
		rjmp	snif_scroll			;we must scroll
		
snif_wdata_06:	cpi	ZL,0x0a				;LF
		brne	snif_wdata_08
		cpi	ZH,0x0d				;CR
		breq	snif_wdata_00			;ignore CR after LF		
		rjmp	snif_wdata_04			;new line	
		
snif_wdata_08:	mov	ZH,r15				;X
		cpi	ZH,0x10				;beyond end?
		brcs	snif_wdata_10			;no
		mov	r15,const_0			;X=0
		inc	r14				;Y+=1
		mov	ZH,r14				;
		cpi	ZH,5
		brcs	snif_wdata_10
		rcall	snif_scroll			;scroll text
;now write char		
snif_wdata_10:	mov	ZL,r13
		rcall	snif_wchar			;write char
		inc	r15				;X+1
		ret
		
;hex mode
snif_wdata_20:	mov	ZH,r15				;X
		cpi	ZH,0x0c				;beyond end?
		brcs	snif_wdata_21			;no
		mov	r15,const_0			;X=0
		inc	r14				;Y+=1
		mov	ZH,r14				;
		cpi	ZH,5
		brcs	snif_wdata_21
		rcall	snif_scroll			;scroll text

snif_wdata_21:	ldi	YH,0x0e				;1 now get char		
		inc	tempreg8			;1 read pointer+1 
		mov	YL,tempreg8			;1
		ld	ZL,Y				;1 this is what we must write
		mov	ZH,r13				;1 old
		mov	r13,ZL				;1 this is our "new old" char
		swap	ZL
		andi	ZL,0x0f
		ldi	ZH,HIGH(snif_hext*2)
		ldi	XL,LOW(snif_hext*2)
		add	ZL,XL
		adc	ZH,const_0
		lpm	ZL,Z				;get hex char
		rcall	snif_wchar			;output
		inc	r15				;next pos
		
		mov	ZL,r13
		andi	ZL,0x0f
		ldi	ZH,HIGH(snif_hext*2)
		ldi	XL,LOW(snif_hext*2)
		add	ZL,XL
		adc	ZH,const_0
		lpm	ZL,Z				;get hex char
		rcall	snif_wchar			;output		
		inc	r15
		inc	r15
		mov	r12,r15				;save pos
		mov	ZH,r15
		cpi	ZH,3
		brne	snif_wdata_22
		ldi	ZH,12				;char 1
		rjmp	snif_wdata_30
snif_wdata_22:	cpi	ZH,6
		brne	snif_wdata_24
		ldi	ZH,13				;char 1
		rjmp	snif_wdata_30
snif_wdata_24:	cpi	ZH,9
		brne	snif_wdata_26
		ldi	ZH,14				;char 1
		rjmp	snif_wdata_30
snif_wdata_26:	ldi	ZH,15

snif_wdata_30:	mov	r15,ZH
		mov	ZL,r13
		rcall	snif_wchar
		mov	r15,r12
		ret				
				
;------------------------------------------------------------------------------
;write char ZL
;------------------------------------------------------------------------------	
snif_wchar:	ldi	XL,LOW(libmio_vram+178)		;1 base address in vram
		ldi	XH,HIGH(libmio_vram+178)	;1			
		ldi	ZH,160				;bytes per char line
		mul	ZH,r14				;*Y
		add	XL,r0				;add Y offset
		adc	XH,r1
		mov	YL,r15				;1 X
		andi	YL,0x0c				;1 mask bits
		lsr	YL				;1 calc /4*3
		mov	ZH,YL				;1
		lsr	YL				;1
		add	YL,ZH				;1
		add	XL,YL				;1 add X offset 
		adc	XH,const_0			;1		
			
		ldi	ZH,HIGH(libmio_ctable*2)	;1 font table
		ldi	YH,0x0f				;1 mirror table
		mov	tempreg2,r15			;1 copy X
		andi	tempreg2,3			;1 mask bits for pos 0-3
		brne	snif_wchar_30			;1/2
		ldi	tempreg2,10			;lines
snif_wchar_21:	lpm	YL,Z				;3 get pixline		
		ld	tempreg1,Y			;2 mirror
		ld	YL,X				;2
		andi	YL,0xc0				;1
		or	YL,tempreg1			;1
		st	X+,YL				;2
		adiw	XL,15				;2
		inc	ZH				;1
		rcall	snif_gbyte			;n serve new byte in queue
		dec	tempreg2			;1
		brne	snif_wchar_21			;1
		ret
		
snif_wchar_30:	cpi	tempreg2,1			;1 mask bits for pos 0-3
		brne	snif_wchar_40			;1/2
		ldi	tempreg2,10			;lines
snif_wchar_31:	lpm	YL,Z				;3 get pixline		
		ld	tempreg1,Y			;2 mirror
		ldi	tempreg3,64			;1 factor
		mul	tempreg1,tempreg3		;2
		ld	YL,X				;2
		andi	YL,0x3f				;1
		or	YL,r0				;1
		st	X+,YL				;2
		ld	YL,X				;2
		andi	YL,0xf0				;1
		or	YL,r1				;1
		st	X+,YL				;2		
		adiw	XL,14				;2
		inc	ZH				;1
		rcall	snif_gbyte			;n serve new byte in queue		
		dec	tempreg2			;1
		brne	snif_wchar_31			;1
		ret			

snif_wchar_40:	cpi	tempreg2,2			;1 mask bits for pos 0-3
		brne	snif_wchar_50			;1/2
		ldi	tempreg2,10			;lines
		adiw	XL,1
snif_wchar_41:	lpm	YL,Z				;3 get pixline		
		ld	tempreg1,Y			;2 mirror
		ldi	tempreg3,16			;1 factor
		mul	tempreg1,tempreg3		;2
		ld	YL,X				;2
		andi	YL,0x0f				;1
		or	YL,r0				;1
		st	X+,YL				;2
		ld	YL,X				;2
		andi	YL,0xfc				;1
		or	YL,r1				;1
		st	X+,YL				;2		
		adiw	XL,14				;2
		inc	ZH				;1
		rcall	snif_gbyte			;n serve new byte in queue		
		dec	tempreg2			;1
		brne	snif_wchar_41			;1
		ret					

snif_wchar_50:	ldi	tempreg2,10			;lines
		adiw	XL,2
snif_wchar_51:	lpm	YL,Z				;3 get pixline		
		ld	tempreg1,Y			;2 mirror
		lsl	tempreg1
		lsl	tempreg1
		ld	YL,X				;2
		andi	YL,0x03				;1
		or	YL,tempreg1			;1
		st	X+,YL				;2
		adiw	XL,15				;2
		inc	ZH				;1
		rcall	snif_gbyte			;n serve new byte in queue		
		dec	tempreg2			;1
		brne	snif_wchar_51			;1
		ret					


;------------------------------------------------------------------------------
;scroll screen
;------------------------------------------------------------------------------			
snif_scroll:	ldi	YL,LOW(libmio_vram+178)		;1 base address in vram
		ldi	YH,HIGH(libmio_vram+178)	;1				
		ldi	ZL,LOW(libmio_vram+338)		;1 base address in vram
		ldi	ZH,HIGH(libmio_vram+338)	;1			
snif_scroll_1:	ldi	r16,40				;lines
snif_scroll_2:	ldi	r17,12				;bytes to move				
snif_scroll_3:	ld	tempreg1,Z+
		st	Y+,tempreg1
		rcall	snif_gbyte			;serve incoming byte
		dec	r17
		brne	snif_scroll_3
		adiw	YL,4
		adiw	ZL,4
		dec	r16
		brne	snif_scroll_2
		
		ldi	r16,10				;lines to clear
snif_scroll_4:	ldi	r17,12				;bytes to move				
snif_scroll_5:	st	Y+,const_0
		rcall	snif_gbyte			;serve incoming byte
		dec	r17
		brne	snif_scroll_5
		adiw	YL,4
		dec	r16
		brne	snif_scroll_4				
		ldi	r16,4				;new Y
		mov	r14,r16		
		mov	r15,const_0			;new X
		ret		
		
snif_brt:	.db	"1200 ",0
		.db	"2400 ",0
		.db	"4800 ",0
		.db	"9600 ",0
		.db	"19200",0
		.db	"31250",0
		.db	"38400",0
		.db	"57600",0
		
snif_hext:	.db	"0123456789abcdef"		
		