;################################################################################
;#										#
;# avr-chipbasic2M - mobile single chip basic computer with ATmega644		#
;# thermometer functions							#
;# copyright (c) 2008 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 2		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;------------------------------------------------------------------------------
;main screen mask
;------------------------------------------------------------------------------
thermo_main:	ldi	YL,LOW(2041)			;corrector value
		ldi	YH,HIGH(2041)
		libeep_read
		sts	libmio_tcorr,tempreg1
		clr	ctrl
		ldi	ZL,LOW(libmio_icons*2+130)
		ldi	ZH,HIGH(libmio_icons*2+130)
		libmio_appframe
		ldi	XL,24
		ldi	XH,0
		ldi	ZL,LOW(thermo_text_1*2)	;text pointer
		ldi	ZH,HIGH(thermo_text_1*2)
		libmio_xztext	
		
thermo_main_2:
		ldi	XL,0x01			;color (white on black)
		libmio_setcolor
		sts	libmio_font,const_1
		libmio_thistext		
		.db 43,24,"MAX:",0,0
		libmio_thistext		
		.db 54,24,"MIN:",0,0		
		sts	libmio_font,const_0				
		libmio_nokey

		rcall	thermo_get0
		cpi	ereg,0
		brne	thermo_err
		sts	bas_array+0,XL		;max
		sts	bas_array+1,XH
		sts	bas_array+2,XL		;min
		sts	bas_array+3,XH
		rjmp	thermo_loop_1

thermo_err:	libmio_oborder
		ldi	XL,31
		ldi	XH,0
		ldi	ZL,LOW(thermo_text_1*2)	;text pointer
		ldi	ZH,HIGH(thermo_text_1*2)
		libmio_xztext	
		ldi	XL,24
		ldi	XH,27
		ldi	ZL,LOW(thermo_text_2*2)	;text pointer
		ldi	ZH,HIGH(thermo_text_2*2)
		libmio_xztext			

thermo_err_2:	libmio_nokey
thermo_err_3:	lds	tempreg1,libmio_keycode
		cpi	tempreg1,0xed		;ESC
		breq	thermo_wloop_2
		cpi	tempreg1,0xea
		brne	thermo_err_3
		rjmp	thermo_main
		
		
thermo_loop:	rcall	thermo_get0
		cpi	ereg,0
		brne	thermo_err
		rcall	thermo_minmax
		
thermo_loop_1:	libmio_thistext
		.db	14,12,0,0
		ldi	ctrl,0x45

		rcall	thermo_vval
		clr	ctrl
		libmio_thistext
		.db	41,48,0,0
		lds	XL,bas_array+0
		lds	XH,bas_array+1
		rcall	thermo_vval
		libmio_thistext
		.db	52,48,0,0
		lds	XL,bas_array+2
		lds	XH,bas_array+3
		rcall	thermo_vval
		
thermo_wloop:	ldi	tempreg3,50
thermo_wloop_1:	lds	tempreg1,libmio_keycode
		cpi	tempreg1,0xed		;ESC
		breq	thermo_wloop_2
		cpi	tempreg1,0xe5		;DOWN
		breq	thermo_wloop_3
		cpi	tempreg1,0xe4		;UP
		brne	thermo_wloop_a
		rjmp	thermo_adj		
thermo_wloop_a:	libmio_sync
		dec	tempreg3
		brne	thermo_wloop_1
		rjmp	thermo_loop
		
thermo_wloop_2:	ret
		
thermo_wloop_3:	lds	XL,bas_array+4
		lds	XH,bas_array+5
		sts	bas_array+0,XL		;max
		sts	bas_array+1,XH
		sts	bas_array+2,XL		;min
		sts	bas_array+3,XH
		ldi	tempreg3,1
		rjmp	thermo_wloop_a		
										
thermo_vval:		
		lsr	XH
		ror	XL
		ror	tempreg3

		libmio_outdez
		libmio_outcomma
		ldi	tempreg1,0x30
		sbrc	tempreg3,7
		ldi	tempreg1,0x35
		libmio_outchar
		libmio_thistext
		.db	255,0x1c,"C",0
		ret		
		
		
thermo_get0:	ldi	tempreg4,0x00		;sensor address
		clr	ereg
		libi2c_rlm75
		cpi	ereg,0
		brne	thermo_get0_1
		lds	tempreg1,libmio_tcorr
		sbiw	XL,8
		add	XL,tempreg1
		adc	XH,const_0
		sts	bas_array+4,XL		;act
		sts	bas_array+5,XH
thermo_get0_1:	ret		
		
		
thermo_minmax:	push	XH
		push	XL
		push	r16
		push	r17
		lds	XL,bas_array+4		;act1
		lds	XH,bas_array+5
		lds	r16,bas_array+0		;max
		lds	r17,bas_array+1
		call	expar_lt
		cpi	XL,1
		brne	thermo_minmax1
		lds	XL,bas_array+4		;act1
		lds	XH,bas_array+5
		sts	bas_array+0,XL		;max
		sts	bas_array+1,XH
thermo_minmax1:	lds	XL,bas_array+4		;act1
		lds	XH,bas_array+5
		lds	r16,bas_array+2		;min
		lds	r17,bas_array+3
		call	expar_gt
		cpi	XL,1
		brne	thermo_minmax2
		lds	XL,bas_array+4		;act1
		lds	XH,bas_array+5
		sts	bas_array+2,XL		;max
		sts	bas_array+3,XH		
thermo_minmax2:	pop	r17
		pop	r16
		pop	XL
		pop	XH
		ret	
		

thermo_adj:	ldi	ZL,LOW(libmio_icons*2+290)
		ldi	ZH,HIGH(libmio_icons*2+290)
		libmio_appframe
		ldi	XL,25
		ldi	XH,0
		ldi	ZL,LOW(thermo_text_3*2)	;text pointer
		ldi	ZH,HIGH(thermo_text_3*2)
		libmio_xztext			
		
		libmio_setnorm
						
thermo_adj_2:	libmio_nokey			;wait for key release
		lds	tempreg1,libmio_tcorr
thermo_adj_3:	andi	tempreg1,0x07
		sts	libmio_tcorr,tempreg1
		ldi	ctrl,0x43
		libmio_thistext
		.db	22,40,"C=",0,0
		lds	XL,libmio_tcorr
		clr	XH		
		subi	XL,8
		sbci	XH,0

		libmio_outdez
thermo_adj_31:	libmio_waitkey
		cpi	tempreg1,0xed		;esc
		brne	thermo_adj_4
		rjmp	thermo_main		;goto thermo menu
thermo_adj_4:	cpi	tempreg1,0xea
		brne	thermo_adj_5
		lds	tempreg1,libmio_tcorr
		ldi	YL,LOW(2041)
		ldi	YH,HIGH(2041)
		libeep_write
		rjmp	thermo_main		;goto thermo menu
		
thermo_adj_5:	cpi	tempreg1,0xe4		;up
		brne	thermo_adj_6
		lds	tempreg1,libmio_tcorr
		inc	tempreg1
		sbrc	tempreg1,4
		ldi	tempreg1,15
		sts	libmio_tcorr,tempreg1
		rjmp	thermo_adj_2
		
thermo_adj_6:	cpi	tempreg1,0xe5		;down
		brne	thermo_adj_7
		lds	tempreg1,libmio_tcorr
		dec	tempreg1
		sbrc	tempreg1,4
		ldi	tempreg1,0
		sts	libmio_tcorr,tempreg1
		rjmp	thermo_adj_2
		
thermo_adj_7:	cpi	tempreg1,0xe2		;left
		breq	thermo_adj_8
		cpi	tempreg1,0xe3		;right
		brne	thermo_adj_31
thermo_adj_8:	ldi	tempreg1,8
		sts	libmio_tcorr,tempreg1
		rjmp	thermo_adj_2					